import * as i0 from '@angular/core';
import { Injectable, Directive, InjectionToken, EventEmitter, Component, Inject, ViewChildren, ViewChild, Input, Output, NgModule } from '@angular/core';
import * as i10 from '@angular/material/select';
import { MatSelectChange, MatSelectModule } from '@angular/material/select';
import * as i12 from '@angular/material/datepicker';
import { MatDatepickerInputEvent, MatDatepickerModule } from '@angular/material/datepicker';
import * as i13 from '@angular/material/checkbox';
import { MatCheckboxChange, MatCheckboxModule } from '@angular/material/checkbox';
import { forkJoin, of, ReplaySubject } from 'rxjs';
import * as i14 from '@angular/material/autocomplete';
import { MatAutocompleteSelectedEvent, MatAutocompleteModule } from '@angular/material/autocomplete';
import * as i1 from '@angular/common/http';
import { HttpHeaders, HttpClientModule } from '@angular/common/http';
import { switchMap, publishReplay, refCount, map, startWith, debounceTime } from 'rxjs/operators';
import jsonata from 'jsonata';
import * as i20 from '@angular/forms';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import * as i22 from '@angular/cdk/drag-drop';
import { moveItemInArray, DragDropModule } from '@angular/cdk/drag-drop';
import { ENTER, COMMA, TAB } from '@angular/cdk/keycodes';
import * as i1$1 from '@angular/material/dialog';
import { MAT_DIALOG_DATA, MatDialogModule } from '@angular/material/dialog';
import * as i4 from '@angular/flex-layout/flex';
import * as i5 from '@angular/flex-layout/extended';
import * as i6 from '@angular/material/icon';
import { MatIconModule } from '@angular/material/icon';
import * as i7 from '@angular/material/tooltip';
import { MatTooltipModule, MAT_TOOLTIP_DEFAULT_OPTIONS } from '@angular/material/tooltip';
import * as i8 from '@angular/material/tabs';
import { MatTabsModule } from '@angular/material/tabs';
import * as i9 from '@angular/material/form-field';
import { MatFormFieldModule } from '@angular/material/form-field';
import * as i11 from '@angular/material/core';
import { MatNativeDateModule } from '@angular/material/core';
import * as i15 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i16 from '@angular/material/input';
import { MatInputModule } from '@angular/material/input';
import * as i17 from '@angular/material/button';
import { MatButtonModule } from '@angular/material/button';
import * as i18 from '@angular/material/expansion';
import { MatExpansionModule } from '@angular/material/expansion';
import * as i19 from '@angular/material/menu';
import { MatMenuModule } from '@angular/material/menu';
import * as i21 from '@angular/material/chips';
import { MatChipsModule } from '@angular/material/chips';
import { MatCardModule } from '@angular/material/card';
import { FlexLayoutModule } from '@angular/flex-layout';
import { MatTableModule } from '@angular/material/table';

/**
 * service for registering custom components
 */
class JsonSchemaFormService {
    constructor() {
        /**
         * layout editor mode
         */
        this.editMode = false;
        /**
         * built-in formats
         */
        this.formats = {
            email: /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/,
            ipv4: /^(?!0)(?!.*\.$)((1?\d?\d|25[0-5]|2[0-4]\d)(\.|$)){4}$/,
            url: /^https?:\/\/(www\.)?[-a-zA-Z0-9@:%._\+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}\b([-a-zA-Z0-9()@:%_\+.~#?&//=]*)$/,
            uri: /^\w+:(\/?\/?)[^\s]+$/
        };
        /**
         * registry of custom widgets. The keys are the values used in schema.widgetType, the values
         * are the Type<any> of the custom widget component implementing WidgetComponent
         */
        this.registry = {};
        /**
         * context data accessible from JSONata expressions
         */
        this.context = {};
        /**
         * registry of displayWith objects
         */
        this.displayWithRegistry = {};
    }
    /**
     * register custom component
     * @param key     the name of the component which is used in schema extension: widget=custom, widgetType=key
     * @param value   the implementation class
     */
    registerComponent(key, value) {
        this.registry[key] = value;
    }
    /**
     * register displayWith implementations
     * @param key     the name of the implementation which is used in schema extension: displayWith=key
     * @param value   the implementation class
     */
    registerDisplayWith(key, value) {
        this.displayWithRegistry[key] = value;
    }
    /**
     * set a context key
     */
    setContext(key, value) {
        this.context[key] = value;
    }
}
JsonSchemaFormService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.1.2", ngImport: i0, type: JsonSchemaFormService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
JsonSchemaFormService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.1.2", ngImport: i0, type: JsonSchemaFormService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.1.2", ngImport: i0, type: JsonSchemaFormService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }] });

/**
 * directive for dynamically loading custom widgets
 */
class WidgetDirective {
    /**
     * allow caller to dynamically insert custom component
     * @param viewContainerRef  dynamic component handle
     */
    constructor(viewContainerRef) {
        this.viewContainerRef = viewContainerRef;
    }
}
WidgetDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.1.2", ngImport: i0, type: WidgetDirective, deps: [{ token: i0.ViewContainerRef }], target: i0.ɵɵFactoryTarget.Directive });
WidgetDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "15.1.2", type: WidgetDirective, selector: "[libWidgetHost]", ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.1.2", ngImport: i0, type: WidgetDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[libWidgetHost]'
                }]
        }], ctorParameters: function () { return [{ type: i0.ViewContainerRef }]; } });

/**
 * static JsonPointer implementation
 */
class JsonPointer {
    /**
     * evaluate the JSON pointer on o
     */
    static jsonPointer(o, pointer) {
        return JsonPointer.jsonPointer2(o, JsonPointer.split(pointer));
    }
    /**
     * evaluate the JSON pointer (parsed array of paths) on o
     */
    static jsonPointer2(o, paths) {
        if (o === undefined) {
            return undefined;
        }
        if (paths.length === 0) {
            return o;
        }
        const path = paths[0];
        const np = Object.assign([], paths);
        np.splice(0, 1);
        if (paths[0] === '*') {
            const res = [];
            for (const f of (typeof (o) === 'object' ? Object.values(o) : o)) {
                res.push(this.jsonPointer2(f, np));
            }
            return res;
        }
        else {
            return this.jsonPointer2(o[path], np);
        }
    }
    /**
     * strip leading / and split the JSON pointer
     */
    static split(s) {
        if (s === '') {
            return [];
        }
        if (s.startsWith('/')) {
            s = s.substring(1);
            const arr = s.split('/');
            for (const a of arr) {
                if (a === '') {
                    throw new Error('JSON Pointer must not contain an empty reference token');
                }
            }
            return arr;
        }
        throw new Error('JSON Pointer must start with /');
    }
}

/**
 * default implementation that handles choices based on schema fields.
 * can be overriden via schema.displayWith
 */
class DefaultChoiceHandler {
    /**
     * create default choice handler
     *
     * @param http      http connection client
     */
    constructor(http) {
        this.http = http;
    }
    /**
     * load choices
     */
    load(value, schema) {
        if (!this.cache) {
            if (schema.choices) {
                // static choices are given, convert them to Choice and merge the result
                const arr = [];
                for (const s of schema.choices) {
                    arr.push(this.choice(s, schema));
                }
                this.cache = forkJoin(arr);
            }
            else if (schema.choicesUrl) {
                // load choices from URL
                this.cache = this.getChoices(schema.choicesUrl, schema.choicesUrlArgs, schema.choicesVerb).pipe(switchMap(res => {
                    if (schema.jsonata) {
                        res = jsonata(schema.jsonata).evaluate(res);
                        if (!Array.isArray(res)) {
                            res = [res];
                            // introduce jsonName, jsonValue
                        }
                    }
                    const obs = [];
                    for (const r of res) {
                        obs.push(this.choice(r, schema));
                    }
                    return forkJoin(obs);
                }), 
                // setup caching
                publishReplay(1), refCount());
            }
        }
        return this.cache;
    }
    /**
     * filter after keystroke
     */
    filter(value, schema, current, choices) {
        return choices.pipe(map(arr => {
            if (!current) {
                return arr;
            }
            const res = arr.filter(i => this.include(i, current));
            return res;
        }));
    }
    /**
     * called from filter, intended to allow subclasses to easily change filter algorithm
     */
    include(i, current) {
        return ('' + i.name).toLowerCase().includes(('' + current).toLowerCase());
    }
    /**
     * default choice implementation: just reuse value as name
     * check for localName
     */
    choice(value, schema) {
        if (schema.displayWith === 'localName') {
            for (const delimiter of ['/', '#', ':', '.']) {
                const parts = value.split(delimiter);
                if (parts.length > 1) {
                    if (parts[parts.length - 1] === '') {
                        return of({ value, name: parts[parts.length - 2] });
                    }
                    else {
                        return of({ value, name: parts[parts.length - 1] });
                    }
                }
            }
            return of({ value, name: value });
        }
        if (schema.jsonata) {
            if (typeof value === 'object') {
                return of(value);
            }
            else {
                // initially, value is a simple string
                return of({ value, name: value });
            }
        }
        if (schema.displayWithChoices) {
            return of({ value, name: schema.displayWithChoices[schema.choices.indexOf(value)] });
        }
        return of({ value, name: value });
    }
    /**
     * handle GET / POST
     */
    getChoices(url, args, verb) {
        if (verb === 'GET') {
            return this.http.get(url, args);
        }
        else {
            return this.http.post(url, args, {
                headers: new HttpHeaders({
                    'Content-Type': 'application/json',
                })
            });
        }
    }
    /**
     * default: no delay
     */
    debounceTime() {
        return 0;
    }
}

/**
 * form editor - has a 1:1 relationship to form node
 */
class Edit {
    /**
     * get access to parent object's fields
     *
     * @param schemaChange  signal a change
     * @param name          the current property name (key)
     * @param schema        the current schema
     * @param parent        the parent's schema (required in order to change the order)
     * @param dialog        dialog service
     */
    constructor(schemaChange, name, schema, parent, dialog, component) {
        this.schemaChange = schemaChange;
        this.name = name;
        this.schema = schema;
        this.parent = parent;
        this.dialog = dialog;
        this.component = component;
    }
    /**
     * can add if addable is not empty
     */
    canAdd() {
        return this.addable().length > 0;
    }
    /**
     * get the entries in "properties" that are not in "order"
     */
    addable() {
        let s;
        if (this.schema.properties) {
            s = this.schema;
        }
        else if (this.schema.items?.properties) {
            s = this.schema.items;
        }
        else if (this.schema.additionalProperties?.properties) {
            s = this.schema.additionalProperties;
        }
        else {
            return [];
        }
        if (s.order) {
            const set = new Set(Object.keys(s.properties));
            for (const p of s.order) {
                if (Array.isArray(p)) {
                    for (const q of p) {
                        set.delete(q);
                    }
                }
                else {
                    set.delete(p);
                }
            }
            return Array.from(set);
        }
        else {
            return [];
        }
    }
    /**
     * add a hidden prop to the end of the list
     */
    addSub(prop) {
        const index = this.parent.order.indexOf(this.name);
        if (index >= 0) {
            this.parent.order[index] = [this.parent.order[index], prop];
        }
        else {
            for (const p of this.parent.order) {
                if (Array.isArray(p)) {
                    if (p.indexOf(this.name) >= 0) {
                        p.push(prop);
                    }
                }
            }
        }
        this.schemaChange.emit();
    }
    /**
     * can add if addable is not empty
     */
    canAddSub() {
        return this.subaddable().length > 0;
    }
    /**
     * get the entries in "properties" that are not in "order"
     */
    subaddable() {
        if (this.parent?.order) {
            const set = new Set(Object.keys(this.parent.properties));
            for (const p of this.parent.order) {
                if (Array.isArray(p)) {
                    for (const q of p) {
                        set.delete(q);
                    }
                }
                else {
                    set.delete(p);
                }
            }
            return Array.from(set);
        }
        else {
            return [];
        }
    }
    /**
     * add a hidden prop to the end of the list
     */
    add(prop) {
        if (this.schema.properties) {
            this.schema.order.push(prop);
        }
        else if (this.schema.items?.properties) {
            this.schema.items.order.push(prop);
        }
        else {
            this.schema.additionalProperties.order.push(prop);
        }
        this.schemaChange.emit();
    }
    /**
     * edit simple schema fields like title and description
     */
    edit() {
        // deep clone object so we have the possibility to cancel editing
        const clone = JSON.parse(JSON.stringify(this.schema));
        // handle (array) example
        if (clone.items?.examples?.length > 0) {
            clone.example = clone.items.examples[0];
        }
        if (clone.examples?.length > 0) {
            clone.example = clone.examples[0];
        }
        // text is default
        if (!clone.widget) {
            clone.widget = 'text';
        }
        // horizontal is default
        if (!clone.layout) {
            clone.layout = 'horizontal';
        }
        // array: apply choices and widget to items
        if (clone.items) {
            clone.widget = clone.items.widget;
            clone.choices = clone.items.choices;
            clone.errorMessage = clone.items.errorMessage;
            clone.format = clone.items.format;
            clone.required = clone.items.required;
            clone.itemlayout = clone.items.layout;
        }
        const dialogRef = this.dialog.open(this.component, { minWidth: '50%', data: clone });
        dialogRef.afterClosed().subscribe(data => {
            if (data) {
                this.schema.title = data.title;
                this.schema.description = data.description;
                this.schema.widget = data.widget;
                this.schema.layout = data.layout;
                this.schema.readOnly = data.readOnly;
                this.schema.errorMessage = data.errorMessage;
                if (data.example) {
                    if (data.items) {
                        this.schema.items.examples = [data.example];
                    }
                    else {
                        this.schema.examples = [data.example];
                    }
                }
                if (data.style) {
                    delete data.style[''];
                }
                if (data.style && Object.keys(data.style).length > 0) {
                    this.schema.style = data.style;
                }
                if (data.class) {
                    data.class = data.class.filter(el => el != null);
                }
                if (data.class?.length > 0) {
                    this.schema.class = data.class;
                }
                if (data.items) {
                    if (data.choices?.length > 0) {
                        this.schema.items.choices = data.choices;
                    }
                    this.schema.items.widget = data.widget;
                    this.schema.items.format = data.format;
                    this.schema.items.errorMessage = data.errorMessage;
                    this.schema.items.required = data.required;
                    this.schema.items.layout = data.itemlayout;
                }
                else {
                    this.schema.choices = data.choices;
                    this.schema.widget = data.widget;
                    this.schema.format = data.format;
                    this.schema.errorMessage = data.errorMessage;
                    this.schema.required = data.required;
                }
                this.schemaChange.emit();
            }
        });
    }
    /**
     * can hide if I my parent is an object
     */
    canHide() {
        return this.parent ? true : false;
    }
    /**
     * hide this prop
     */
    hide() {
        if (!this.parent.order) {
            this.parent.order = Object.keys(this.parent.properties);
        }
        let index = 0;
        for (const p of this.parent.order) {
            if (Array.isArray(p)) {
                if (p.indexOf(this.name) >= 0) {
                    p.splice(p.indexOf(this.name), 1);
                    if (p.length === 1) {
                        this.parent.order[index] = p[0];
                    }
                }
            }
            index++;
        }
        if (this.parent.order.indexOf(this.name) >= 0) {
            this.parent.order.splice(this.parent.order.indexOf(this.name), 1);
        }
        this.schemaChange.emit();
    }
    /**
     * can I go up
     */
    canUp() {
        if (!this.parent) {
            return false;
        }
        const props = this.parent.order ? this.parent.order : Object.keys(this.parent.properties);
        const index = props.indexOf(this.name);
        if (index >= 0) {
            return index > 0;
        }
        else {
            for (const p of props) {
                if (Array.isArray(p)) {
                    if (p.indexOf(this.name) >= 0) {
                        return p.indexOf(this.name) > 0;
                    }
                }
            }
        }
    }
    /**
     * go up
     */
    up() {
        if (!this.parent.order) {
            this.parent.order = Object.keys(this.parent.properties);
        }
        let index = this.parent.order.indexOf(this.name);
        if (index >= 0) {
            const tmp = this.parent.order[index - 1];
            this.parent.order[index - 1] = this.parent.order[index];
            this.parent.order[index] = tmp;
        }
        else {
            for (const p of this.parent.order) {
                if (Array.isArray(p)) {
                    index = p.indexOf(this.name);
                    if (index >= 0) {
                        const tmp = p[index - 1];
                        p[index - 1] = p[index];
                        p[index] = tmp;
                    }
                }
            }
        }
        this.schemaChange.emit();
    }
    /**
     * can I go down
     */
    canDown() {
        if (!this.parent) {
            return false;
        }
        const props = this.parent.order ? this.parent.order : Object.keys(this.parent.properties);
        const index = props.indexOf(this.name);
        if (index >= 0) {
            return index < props.length - 1;
        }
        else {
            for (const p of props) {
                if (Array.isArray(p)) {
                    if (p.indexOf(this.name) >= 0) {
                        return p.indexOf(this.name) < p.length - 1;
                    }
                }
            }
        }
    }
    /**
     * go down
     */
    down() {
        if (!this.parent.order) {
            this.parent.order = Object.keys(this.parent.properties);
        }
        let index = this.parent.order.indexOf(this.name);
        if (index >= 0) {
            const tmp = this.parent.order[index + 1];
            this.parent.order[index + 1] = this.parent.order[index];
            this.parent.order[index] = tmp;
        }
        else {
            for (const p of this.parent.order) {
                if (Array.isArray(p)) {
                    index = p.indexOf(this.name);
                    if (index >= 0) {
                        const tmp = p[index + 1];
                        p[index + 1] = p[index];
                        p[index] = tmp;
                    }
                }
            }
        }
        this.schemaChange.emit();
    }
}

const EDIT_DIALOG_TOKEN = new InjectionToken('EDIT_DIALOG_TOKEN');

/**
 * generates an input form base on JSON schema and JSON object.
 * The component is used recursively.
 */
class JsonSchemaFormComponent {
    /**
     * component constructor
     * @param http                        http client
     * @param componentFactoryResolver    allows dynamic components
     * @param service                     application service for registering components etc.
     * @param dialog                      dialog service
     */
    constructor(http, componentFactoryResolver, service, dialog, component) {
        this.http = http;
        this.componentFactoryResolver = componentFactoryResolver;
        this.service = service;
        this.dialog = dialog;
        this.component = component;
        /**
         * emit changes done by the user in the component
         */
        this.valueChange = new EventEmitter();
        /**
         * emit whether this part of the form is valid
         */
        this.errorChange = new EventEmitter();
        /**
         * indicate schema changes done via the layout editor
         */
        this.schemaChange = new EventEmitter();
        /**
         * indicates whether this is the root of the component tree
         */
        this.isRoot = false;
        /**
         * avoids change detection issues for arrays
         */
        this.arrayIndices = [];
        /**
         * avoids change detection issues for arrays
         */
        this.additionalPropNames = [];
        /**
         * complete chip entry with enter or comma
         */
        this.separatorKeysCodes = [ENTER, COMMA, TAB];
        /**
         * angular pipe sorting function for keyValue - keep the JSON order and do not
         * order alphabetically
         */
        this.originalOrder = (a, b) => {
            return 0;
        };
    }
    /**
     * apply order, called anytime properties are set
     */
    setOrderedProperties() {
        if (this.schema.order) {
            this.orderedProperties = [];
            for (const p of this.schema.order) {
                const arr = Array.isArray(p) ? p : [p];
                const o = {};
                for (const q of arr) {
                    o[q] = this.schema.properties[q];
                }
                this.orderedProperties.push(o);
            }
        }
        else if (this.schema.properties) {
            this.orderedProperties = [];
            for (const [key, value] of Object.entries(this.schema.properties)) {
                const o = {};
                o[key] = value;
                this.orderedProperties.push(o);
            }
        }
    }
    /**
     * initialize the comonent.
     * replace undefined with null and init autocomplete choices
     */
    ngOnInit() {
        if (!this.schema) {
            this.schema = { type: 'string' };
        }
        this.readOnly = this.schema.readOnly || (this.schema.createOnly && this.value);
        if (!this.rootSchema) {
            this.rootSchema = this.schema;
            this.rootValue = this.value;
            this.isRoot = true;
        }
        if (!this.schema.type) {
            const p = this.schema.$ref;
            const parts = p.split('#');
            if (parts.length === 1) {
                // URL only
                this.url(parts[0], null);
            }
            else {
                if (parts[0]) {
                    // URL + anchor
                    this.url(parts[0], parts[1]);
                }
                else {
                    // local ref
                    this.schema = JsonPointer.jsonPointer(this.rootSchema, parts[1]);
                    this.setOrderedProperties();
                }
            }
        }
        else {
            this.setOrderedProperties();
        }
        if (typeof this.value === 'undefined') {
            if (this.schema.default) {
                this.value = this.schema.default;
                setTimeout(() => this.emit(this.value), 500);
            }
            else {
                if (!this.hideUndefined) {
                    this.value = null;
                }
            }
        }
        if (this.getLayout() === 'custom') {
            this.loadComponent();
        }
        if (this.isRoot) {
            setTimeout(() => {
                this.errorChange.emit(this.recursiveError());
            }, 10);
        }
        this.ch = this.service.displayWithRegistry[this.schema.displayWith];
        if (!this.ch) {
            this.ch = new DefaultChoiceHandler(this.http);
        }
        this.control = new FormControl(this.value);
        this.choices = new ReplaySubject();
        if (Array.isArray(this.value)) {
            const arr = [];
            for (const i of this.value) {
                arr.push({ name: i, value: i });
            }
            this.choices.next(arr);
        }
        else {
            this.choices.next([{ name: this.value, value: this.value }]);
        }
        if (this.value || this.value === 0) {
            if (Array.isArray(this.value)) {
                this.arrayIndices = Array.from(Array(this.value.length).keys());
                const arr = [];
                for (const i of this.value) {
                    arr.push(this.ch.choice(i, this.schema));
                }
                forkJoin(arr).subscribe(res => this.choices.next(res));
            }
            else {
                this.ch.choice(this.value, this.schema).subscribe(res => this.choices.next([res]));
            }
        }
        this.filteredOptions = this.control.valueChanges
            .pipe(startWith(this.value), debounceTime(this.ch.debounceTime()), switchMap(x => {
            this.change({ target: { value: x } });
            return this.ch.filter(this.value, this.schema, x, this.choices);
        }));
        if (this.schema.choicesLoad === 'onLoad') {
            this.focus();
        }
        if (this.schema.additionalProperties && this.value) {
            this.additionalPropNames = Object.keys(this.value);
            this.arrayIndices = Array.from(Array(this.additionalPropNames.length).keys());
        }
        this.edit = new Edit(this.schemaChange, this.name, this.schema, this.parentSchema, this.dialog, this.component);
    }
    /**
     * choice element activated - load values
     */
    focus() {
        const o = this.ch.load(this.value, this.schema);
        if (o) {
            o.subscribe(res => {
                this.choices.next(res);
            });
        }
    }
    /**
     * load schema from ref, apply pointer if needed
     */
    url(ref, pointer) {
        // URL + anchor
        this.base = this.base ? new URL(ref, this.base).href : ref;
        // check root schema referenced map
        if (this.rootSchema.referenced && this.rootSchema.referenced[this.base]) {
            const res = this.rootSchema.referenced[this.base];
            this.schema = pointer ? JsonPointer.jsonPointer(res, pointer) : res;
            this.setOrderedProperties();
            return;
        }
        this.http.get(this.base).subscribe(res => {
            this.schema = pointer ? JsonPointer.jsonPointer(res, pointer) : res;
            this.setOrderedProperties();
        }, error => console.log(error));
        // set temporary pseudo schema
        this.schema = { type: 'string' };
        this.setOrderedProperties();
    }
    /**
     * emit valueChange event and also any validation error
     */
    emit(event) {
        this.valueChange.emit(event);
        if (this.isRoot) {
            setTimeout(() => {
                this.errorChange.emit(this.recursiveError());
            }, 10);
        }
    }
    /**
     * if the schema changes from the outside,
     * reset the component state wrt. errors and the choices cache
     */
    ngOnChanges(changes) {
        if (changes.value && !changes.value.isFirstChange() && this.getLayout() === 'autocomplete') {
            // make sure autocomplete form control picks up external changes
            this.control.setValue(changes.value.currentValue);
        }
        if (changes.schema) {
            if (changes.schema.previousValue) {
                this.rootSchema = null;
                if (this.widgetHost.viewContainerRef) {
                    this.widgetHost.viewContainerRef.clear();
                }
                this.ngOnInit();
            }
        }
        if (changes.switch && !changes.switch.isFirstChange()) {
            if (this.getLayout() === 'custom') {
                this.loadComponent();
            }
            else {
                if (this.widgetHost.viewContainerRef) {
                    this.widgetHost.viewContainerRef.clear();
                }
            }
        }
    }
    /**
     * key method to instruct the display which HTML block to activate.
     */
    getLayout() {
        if (this.schema.case && this.schema.case.indexOf(this.switch) < 0) {
            return 'none';
        }
        if (this.schema.widget === 'custom') {
            return 'custom';
        }
        if (this.hideUndefined && this.value === undefined) {
            return 'none';
        }
        if (this.schema.widget === 'upload') {
            return 'upload';
        }
        if (this.schema.widget === 'upload64') {
            return 'upload64';
        }
        if (this.schema.type === 'object') {
            if (this.schema.additionalProperties) {
                if (this.schema.layout === 'tab') {
                    return 'additionalPropertiesTab';
                }
                return 'additionalProperties';
            }
            return 'object';
        }
        if (this.schema.type === 'array') {
            if (this.schema.layout === 'tab') {
                return 'tab';
            }
            if (this.schema.layout === 'table') {
                return 'table';
            }
            if (this.schema.layout === 'select') {
                return 'array-select';
            }
            if (this.schema.layout === 'chips') {
                return 'chips';
            }
            return 'array';
        }
        if (this.schema.enum) {
            return 'enum';
        }
        if (this.schema.widget === 'date') {
            return 'date';
        }
        if (this.schema.widget === 'textarea') {
            return 'textarea';
        }
        if (this.schema.type === 'boolean') {
            return 'checkbox';
        }
        if (this.schema.choicesUrl) {
            return 'autocomplete';
        }
        if (this.schema.choices) {
            return 'autocomplete';
        }
        if (this.schema.displayWith) {
            return 'autocomplete';
        }
        return 'single';
    }
    /**
     * called from template in the "simple" type. If "type" is "number" or "integer",
     * the HTML input type is "number" which avoids normal string input
     */
    getInputType(schema) {
        if (schema.type === 'number') {
            return 'number';
        }
        if (schema.type === 'integer') {
            return 'number';
        }
        return schema.widget;
    }
    /**
     * called from template in the "simple" type. Step will defaults to `any`
     * otherwise it will use the configured value in the schema.
     * Allows fine control over validation and increment/decrement of the input
     *
     * @param schema JSON schema object describing the property
     *
     * @see https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#attr-step
     */
    getStepValue(schema) {
        return schema.multipleOf ? '' + schema.multipleOf : "any";
    }
    /**
     * event handler for object display. Catches the child component event and
     * handle it by setting the value[key].
     * Also init null objects with {}
     */
    onValueChange(key, value) {
        if (!this.value || typeof (this.value) !== 'object') {
            this.value = {};
        }
        this.value[key] = value;
        if (this.schema.computed) {
            for (const field of Object.keys(this.schema.computed)) {
                const expression = this.schema.computed[field];
                const expr = jsonata(expression);
                expr.registerFunction('context', (k) => this.service.context[k], '<s:j>');
                this.value[field] = expr.evaluate(this.value);
            }
        }
        this.emit(this.value);
    }
    /**
     * add an element to an array.
     * null arrays are initialized with []
     */
    add() {
        if (!(this.value instanceof Array)) {
            this.value = [];
        }
        if (this.schema.items?.type === 'array') {
            this.value.push([]);
        }
        else if (this.schema.items?.type === 'object') {
            this.value.push({});
        }
        else {
            this.value.push(null);
        }
        this.arrayIndices.push(this.value.length - 1);
        this.emit(this.value);
    }
    /**
     * event handler for adding a field
     */
    addField() {
        if (!this.value) {
            this.value = {};
        }
        if (Object.keys(this.value).includes('')) {
            return;
        }
        this.value[''] = null;
        this.additionalPropNames.push('');
        this.arrayIndices.push(this.additionalPropNames.length - 1);
        this.emit(this.value);
    }
    /**
     * remove an element from an array
     */
    remove(i) {
        this.value.splice(i, 1);
        this.arrayIndices.pop();
        this.emit(this.value);
    }
    /**
     * remove a field
     */
    removeField(key) {
        delete this.value[key];
        this.additionalPropNames.splice(this.additionalPropNames.indexOf(key), 1);
        this.arrayIndices.pop();
        this.emit(this.value);
    }
    /**
     * event handler for changed field names with "additionalProperties"
     */
    fieldNameChange(key, newvalue) {
        if (this.additionalPropNames.includes(newvalue)) {
            const tmp = this.additionalPropNames;
            this.additionalPropNames = [];
            setTimeout(() => {
                this.additionalPropNames = tmp;
            }, 1);
            return;
        }
        this.value[newvalue] = this.value[key];
        delete this.value[key];
        this.additionalPropNames[this.additionalPropNames.indexOf(key)] = newvalue;
        this.emit(this.value);
    }
    /**
     * returns the validation error on this level and call recursively for all children.
     * returns null if the form contents is valid
     */
    recursiveError() {
        const e = this.error();
        if (e) {
            return e;
        }
        if (this.child) {
            return this.child.recursiveError();
        }
        if (this.children) {
            for (const c of this.children) {
                const r = c.recursiveError();
                if (r) {
                    return r;
                }
            }
        }
        return null;
    }
    /**
     * return the error message provided in the schema or the generic error message
     * returned from the validation code
     */
    e(error) {
        if (this.schema.errorMessage) {
            return this.schema.errorMessage;
        }
        return error;
    }
    /**
     * return error string
     */
    error() {
        if (this.schema.widget === 'custom') {
            return this.customError;
        }
        if (this.schema.case && this.schema.case.indexOf(this.switch) < 0) {
            return null;
        }
        if (this.value || this.value === 0) {
            if (this.schema.maxItems || this.schema.maxItems === 0) {
                if (!(this.value.length <= this.schema.maxItems)) {
                    return this.e('Only ' + this.schema.maxItems + ' array entries allowed');
                }
            }
            if (this.schema.uniqueItems) {
                if (!(new Set(this.value).size === this.value.length)) {
                    return this.e('Array entries must be unique');
                }
            }
            if (this.schema.minItems || this.schema.minItems === 0) {
                if (!(this.value.length >= this.schema.minItems)) {
                    return this.e('At least ' + this.schema.minItems + ' array entries required');
                }
            }
            if (this.schema.maxProperties || this.schema.maxProperties === 0) {
                if (!(Object.keys(this.value).length <= this.schema.maxProperties)) {
                    return this.e('Only ' + this.schema.maxProperties + ' fields allowed');
                }
            }
            if (this.schema.propertyNames) {
                for (const key of Object.keys(this.value)) {
                    const re = new RegExp(this.schema.propertyNames);
                    if (!re.test(key)) {
                        return this.e('illegal field name: ' + key);
                    }
                }
            }
            if (this.schema.dependencies) {
                for (const dep of Object.keys(this.schema.dependencies)) {
                    if (this.value[dep]) {
                        for (const l of this.schema.dependencies[dep]) {
                            if (!this.value[l]) {
                                return this.e(dep + ' depends on ' + l);
                            }
                        }
                    }
                }
            }
            if (this.schema.minProperties || this.schema.minProperties === 0) {
                if (!(Object.keys(this.value).length >= this.schema.minProperties)) {
                    return this.e('At least ' + this.schema.minProperties + ' fields required');
                }
            }
            if (this.schema.maxLength || this.schema.maxLength === 0) {
                if (!(('' + this.value).length <= this.schema.maxLength)) {
                    return this.e('Input is longer than ' + this.schema.maxLength);
                }
            }
            if (this.schema.minLength || this.schema.minLength === 0) {
                if (!(('' + this.value).length >= this.schema.minLength)) {
                    return this.e('Input is shorter than ' + this.schema.minLength);
                }
            }
            if (this.schema.multipleOf || this.schema.multipleOf === 0) {
                if (!Number.isInteger(Number(this.value) / this.schema.multipleOf)) {
                    return this.e('Must be multiple of ' + this.schema.multipleOf);
                }
            }
            if (this.schema.exclusiveMaximum || this.schema.exclusiveMaximum === 0) {
                if (!(Number(this.value) < this.schema.exclusiveMaximum)) {
                    return this.e('Must be less than ' + this.schema.exclusiveMaximum);
                }
            }
            if (this.schema.maximum || this.schema.maximum === 0) {
                if (!(Number(this.value) <= this.schema.maximum)) {
                    return this.e('Must be less than or equal ' + this.schema.maximum);
                }
            }
            if (this.schema.exclusiveMinimum || this.schema.exclusiveMinimum === 0) {
                if (!(Number(this.value) > this.schema.exclusiveMinimum)) {
                    return this.e('Must greater than ' + this.schema.exclusiveMinimum);
                }
            }
            if (this.schema.minimum || this.schema.minimum === 0) {
                if (!(Number(this.value) >= this.schema.minimum)) {
                    return this.e('Must greater than or equal ' + this.schema.minimum);
                }
            }
        }
        if (this.required) {
            if (this.value == null || Object.is(this.value, NaN)) {
                return this.e('required');
            }
        }
        if (this.schema.required) {
            for (const dep of this.schema.required) {
                if (!this.value[dep] && this.value[dep] !== false && this.value[dep] !== 0) {
                    // ignore 'required' if dep is inactive due to switch / case
                    let inactive = false;
                    if (this.schema.switch) {
                        const switc = this.value[this.schema.switch];
                        if (switc && this.schema.properties[dep].case?.indexOf(switc) < 0) {
                            inactive = true;
                        }
                    }
                    if (!inactive) {
                        return this.e(dep + ' is required');
                    }
                }
            }
        }
        if (this.schema.pattern) {
            const re = new RegExp(this.schema.pattern);
            if (this.value && !re.test(this.value)) {
                return this.e('illegal string');
            }
        }
        if (this.schema.format && this.service.formats[this.schema.format]) {
            const re = new RegExp(this.service.formats[this.schema.format]);
            if (this.value && !re.test(this.value)) {
                return this.e('illegal string');
            }
        }
        return null;
    }
    /**
     * use the element title if present, defaults to the label input or "" is both are null
     */
    getLabel() {
        if (this.schema.title) {
            return this.schema.title;
        }
        if (this.label) {
            return this.label;
        }
        return '';
    }
    /**
     * input element change handler.
     * normalize the different kind of events, handle the datatypes, set the value and emit the change
     */
    change(event) {
        let eventTarget;
        if (event instanceof MatSelectChange) {
            event = event.value;
        }
        else if (event instanceof MatDatepickerInputEvent) {
            event = this.serializeDate(event.value, this.schema.dateFormat, this.schema.type);
        }
        else if (event instanceof MatAutocompleteSelectedEvent) {
            event = event.option.value;
        }
        else if (event instanceof MatCheckboxChange) {
            event = event.checked;
        }
        else {
            // save the event target in case the parsing changes the value
            // (e.g. integer input 5.3 becomes 5, this is reflected on the UI via this handle)
            eventTarget = event.target;
            event = event.target.value;
        }
        if (event === '') {
            event = null;
        }
        if (event == null) {
            this.value = null;
        }
        if (this.schema.type === 'number') {
            this.value = isNaN(parseFloat(event)) ? event : parseFloat(event);
        }
        else if (this.schema.type === 'integer') {
            this.value = isNaN(parseInt(event)) ? event : parseInt(event, 10);
        }
        else if (this.schema.type === 'boolean') {
            if (typeof event === 'string') {
                if (event === 'true') {
                    this.value = true;
                }
                else if (event === 'false') {
                    this.value = false;
                }
                else {
                    this.value = null;
                }
            }
            else {
                this.value = event;
            }
        }
        else if (this.schema.type === 'string') {
            this.value = event;
        }
        else if (this.schema.type === 'array') {
            this.value = event;
        }
        else {
            throw new Error('unknown type: ' + this.schema.type);
        }
        this.emit(this.value);
    }
    /**
     * allows for the result of a file upload to be written into a text form element
     */
    handleFileInput(base64, event) {
        if (10 * 1024 * 1024 <= event.target.files.item(0).size) {
            console.log('The file size is limited to 10MB');
            return;
        }
        const reader = new FileReader();
        reader.onload = () => {
            this.value = reader.result;
            if (this.schema.type === 'array') {
                this.value = JSON.parse(this.value);
            }
            if (this.schema.type === 'object') {
                try {
                    this.value = JSON.parse(this.value);
                }
                catch (ignore) {
                }
                this.value = {
                    name: event.target.files.item(0).name,
                    lastModified: event.target.files.item(0).lastModified,
                    size: event.target.files.item(0).size,
                    type: event.target.files.item(0).type,
                    value: this.value
                };
            }
            this.emit(this.value);
        };
        if (base64) {
            reader.readAsDataURL(event.target.files.item(0));
        }
        else {
            reader.readAsText(event.target.files.item(0));
        }
    }
    /**
     * get example values from example array and default
     */
    example() {
        if (this.schema.examples && this.schema.examples[0]) {
            return this.schema.examples[0];
        }
        if (this.schema.default) {
            return this.schema.default;
        }
        return null;
    }
    /**
     * load the dynamic custom widget
     */
    loadComponent() {
        const componentFactory = this.componentFactoryResolver.resolveComponentFactory(this.service.registry[this.schema.widgetType]);
        const viewContainerRef = this.widgetHost.viewContainerRef;
        viewContainerRef.clear();
        const componentRef = viewContainerRef.createComponent(componentFactory);
        // input values
        componentRef.instance.label = this.label;
        componentRef.instance.value = this.value;
        componentRef.instance.schema = this.schema;
        componentRef.instance.rootSchema = this.rootSchema;
        componentRef.instance.rootValue = this.rootValue;
        // subscribe to value changes and forward them
        componentRef.instance.valueChange.subscribe(data => {
            this.value = data;
            this.emit(this.value);
        });
        // subscribe to error changes and forward them
        componentRef.instance.errorChange.subscribe(error => {
            this.customError = error;
            this.errorChange.emit(error);
        });
    }
    /**
     * used for expansion panels - set value and forward event
     */
    setAndEmit(event) {
        this.value = event;
        this.emit(this.value);
    }
    /**
     * set an array element and emit value change event
     */
    setIndexAndEmit(i, event) {
        this.value[i] = event;
        this.emit(this.value);
    }
    /**
     * set an array element's field and emit value change event (applies to table layout)
     */
    setIndexAndEmitTable(i, field, event) {
        this.value[i][field] = event;
        this.emit(this.value);
    }
    /**
     * used when hideUndefined is active. Called from the UI to
     * show a property with undefined value (in order to be able to set if in the form)
     */
    showProperty(prop) {
        if (!this.value) {
            this.value = {};
        }
        if (this.value[prop] === undefined) {
            this.value[prop] = null;
        }
        else if (this.value[prop] === null) {
            this.value[prop] = undefined;
        }
    }
    /**
     * used when hideUndefined is active. Called from the UI
     * to determine which properties are included in the "to add" list
     */
    showPropertyList() {
        if (this.schema.switch && this.value) {
            const sw = this.value[this.schema.switch];
            const props = [];
            for (const [k, v] of Object.entries(this.schema.properties)) {
                if (v.case) {
                    if (v.case.includes(sw)) {
                        props.push(k);
                    }
                }
                else {
                    props.push(k);
                }
            }
            return props.sort();
        }
        else {
            return Object.keys(this.schema.properties).sort();
        }
    }
    /**
     * string to date
     * @param date    date string / number (millisecs since 1970)
     * @param format  date format
     */
    parseDate(date, format) {
        if (!date && date !== 0) {
            return date;
        }
        if (typeof date === 'number') {
            return this.sameDate(new Date(date));
        }
        if (!format) {
            return date;
        }
        const pdate = date.split(this.getDelimiter(format));
        const pformat = format.split(this.getDelimiter(format));
        return this.sameDate(new Date(pdate[pformat.indexOf('yyyy')], pdate[pformat.indexOf('MM')] - 1, pdate[pformat.indexOf('dd')]));
    }
    /**
     * make sure to return the same date object instance (cannot delete date #83)
     */
    sameDate(nd) {
        if (!this.date) {
            this.date = nd;
        }
        if (this.date.getTime() !== nd.getTime()) {
            this.date = nd;
        }
        return this.date;
    }
    /**
     * date to string
     * @param date    the date to serialize
     * @param format  the date format (e.g. dd-MM-yyyy)
     * @param type    target datatype (allows serializing to millisecs since 1970)
     */
    serializeDate(date, format, type) {
        if (date == null) {
            return '';
        }
        if (type === 'integer' || type === 'number') {
            return '' + date.valueOf();
        }
        if (!format) {
            return date.toISOString();
        }
        const pformat = format.split(this.getDelimiter(format));
        const pdate = [null, null, null];
        pdate[pformat.indexOf('yyyy')] = date.getFullYear();
        pdate[pformat.indexOf('MM')] = date.getMonth() + 1;
        pdate[pformat.indexOf('dd')] = date.getDate();
        return pdate[0] + this.getDelimiter(format) + pdate[1] + this.getDelimiter(format) + pdate[2];
    }
    /**
     * find the first non letter character in a date format such as dd/MM/yyyy (returns /)
     */
    getDelimiter(format) {
        const delim = format.match(/\W/g);
        if (!delim[0]) {
            throw new Error('No delimiter found in date format: ' + format);
        }
        return delim[0];
    }
    /**
     * new chip entered
     */
    addChip(event) {
        const input = event.input;
        const value = event.value;
        // Add our fruit
        if ((value || '').trim()) {
            if (!this.value) {
                this.value = [];
            }
            this.value.push(value.trim());
            this.emit(this.value);
        }
        // Reset the input value
        if (input) {
            input.value = '';
        }
    }
    /**
     * remove a chip
     */
    removeChip(v) {
        const index = this.value.indexOf(v);
        if (index >= 0) {
            this.value.splice(index, 1);
            if (this.value.length === 0) {
                this.value = null;
            }
            this.emit(this.value);
        }
    }
    /**
     * chips d&d handler
     */
    dropChip(event) {
        moveItemInArray(this.value, event.previousIndex, event.currentIndex);
        this.emit(this.value);
    }
}
JsonSchemaFormComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.1.2", ngImport: i0, type: JsonSchemaFormComponent, deps: [{ token: i1.HttpClient }, { token: i0.ComponentFactoryResolver }, { token: JsonSchemaFormService }, { token: i1$1.MatDialog }, { token: EDIT_DIALOG_TOKEN }], target: i0.ɵɵFactoryTarget.Component });
JsonSchemaFormComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.1.2", type: JsonSchemaFormComponent, selector: "lib-json-schema-form", inputs: { name: "name", label: "label", value: "value", rootValue: "rootValue", schema: "schema", parentSchema: "parentSchema", rootSchema: "rootSchema", base: "base", switch: "switch", hideUndefined: "hideUndefined", inExpansion: "inExpansion", inArray: "inArray", required: "required" }, outputs: { valueChange: "valueChange", errorChange: "errorChange", schemaChange: "schemaChange" }, viewQueries: [{ propertyName: "child", first: true, predicate: ["child"], descendants: true }, { propertyName: "widgetHost", first: true, predicate: WidgetDirective, descendants: true, static: true }, { propertyName: "children", predicate: ["children"], descendants: true }], usesOnChanges: true, ngImport: i0, template: "<mat-expansion-panel *ngIf=\"(schema.expanded === true || schema.expanded === false) && !inExpansion\"\r\n    [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\" [expanded]=\"schema.expanded\">\r\n    <mat-expansion-panel-header>\r\n        <mat-panel-title>{{label}}</mat-panel-title>\r\n        <mat-panel-description>{{schema.description}}</mat-panel-description>\r\n    </mat-expansion-panel-header>\r\n    <lib-json-schema-form #child [label]=\"label\" [name]=\"name\" [value]=\"value\" (valueChange)=\"setAndEmit($event)\"\r\n        [switch]=\"switch\" [rootValue]=\"rootValue\" [rootSchema]=\"rootSchema\" [schema]=\"schema\" [inExpansion]=\"true\"\r\n        [base]=\"base\" (schemaChange)=\"schemaChange.emit()\">\r\n    </lib-json-schema-form>\r\n</mat-expansion-panel>\r\n\r\n<div *ngIf=\"(schema.expanded === null || schema.expanded === undefined) || inExpansion\"\r\n    [matTooltip]=\"schema.description\" role=\"form\">\r\n\r\n    <div *ngIf=\"service.editMode && !schema.static && !inArray && getLayout() !== 'none'\">\r\n        <mat-icon style=\"cursor: pointer\" [matMenuTriggerFor]=\"menu\"\r\n            [matTooltip]=\"name ? 'Form element '+name : (isRoot ? 'Form root' : '')\">\r\n            more_vert\r\n        </mat-icon>\r\n\r\n        <mat-menu #menu=\"matMenu\">\r\n            <button mat-menu-item (click)=\"edit.edit()\">\r\n                <mat-icon>edit</mat-icon>Edit\r\n            </button>\r\n            <button [disabled]=\"!edit.canAdd()\" mat-menu-item [matMenuTriggerFor]=\"madd\">\r\n                <mat-icon>add</mat-icon>Show\r\n            </button>\r\n            <button [disabled]=\"!edit.canAddSub()\" mat-menu-item [matMenuTriggerFor]=\"madd2\">\r\n                <mat-icon>playlist_add</mat-icon>Show inline\r\n            </button>\r\n            <button [disabled]=\" !edit.canHide()\" mat-menu-item (click)=\"edit.hide()\">\r\n                <mat-icon>remove</mat-icon>Hide\r\n            </button>\r\n            <button [disabled]=\"!edit.canUp()\" mat-menu-item (click)=\"edit.up()\">\r\n                <mat-icon>north_west</mat-icon>Up / Left\r\n            </button>\r\n            <button [disabled]=\"!edit.canDown()\" mat-menu-item (click)=\"edit.down()\">\r\n                <mat-icon>south_east</mat-icon>Down / Right\r\n            </button>\r\n        </mat-menu>\r\n\r\n        <mat-menu #madd=\"matMenu\">\r\n            <button *ngFor=\"let x of edit.addable()\" mat-menu-item (click)=\"edit.add(x)\">{{x}}</button>\r\n        </mat-menu>\r\n\r\n        <mat-menu #madd2=\"matMenu\">\r\n            <button *ngFor=\"let x of edit.subaddable()\" mat-menu-item (click)=\"edit.addSub(x)\">{{x}}</button>\r\n        </mat-menu>\r\n    </div>\r\n\r\n    <!-- object row wrap layout -->\r\n    <span *ngIf=\"getLayout() === 'object'\" [fxLayout]=\"schema.layout === 'vertical' ? 'column' : 'row wrap'\"\r\n        [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n        <div *ngFor=\"let op of orderedProperties\" [fxLayout]=\"schema.layout !== 'vertical' ? 'column' : 'row wrap'\">\r\n            <div *ngFor=\" let x of op | keyvalue:originalOrder\" style=\"display: flex; margin: 2px\">\r\n                <div *ngIf=\"x.value && (x.value.type || x.value.$ref)\">\r\n                    <lib-json-schema-form #children [name]=\"x.key\" [label]=\"x.value.title ? x.value.title : x.key\"\r\n                        [value]=\"value ? value[x.key] : undefined\" (valueChange)=\"onValueChange(x.key, $event)\"\r\n                        [switch]=\"value ? value[schema.switch] : undefined\" [rootValue]=\"rootValue\"\r\n                        [rootSchema]=\"rootSchema\" [schema]=\"x.value\" [parentSchema]=\"schema\"\r\n                        [required]=\"schema.required ? schema.required.includes(x.key) : false\"\r\n                        [hideUndefined]=\"schema.hideUndefined\" [base]=\"base\" (schemaChange)=\"schemaChange.emit()\">\r\n                    </lib-json-schema-form>\r\n                </div>\r\n            </div>\r\n        </div>\r\n        <div *ngIf=\"schema.hideUndefined\" style=\"margin-right: 20px;\">\r\n            <button mat-icon-button [matMenuTriggerFor]=\"menu\">\r\n                <mat-icon>more_vert</mat-icon>\r\n            </button>\r\n            <mat-menu #menu=\"matMenu\">\r\n                <div *ngFor=\"let x of showPropertyList()\">\r\n                    <mat-checkbox [disabled]=\"value ? value[x] : false\" [checked]=\"value && (value[x] !== undefined)\"\r\n                        (change)=\"showProperty(x)\">\r\n                        {{x}}\r\n                    </mat-checkbox>\r\n                </div>\r\n            </mat-menu>\r\n        </div>\r\n    </span>\r\n\r\n    <!-- additional properties -->\r\n    <span *ngIf=\"getLayout() === 'additionalProperties'\"\r\n        [fxLayout]=\"schema.layout === 'vertical' ? 'column' : 'row wrap'\" [ngStyle]=\"schema.style\"\r\n        [ngClass]=\"schema.class\">\r\n        <div *ngFor=\"let i of arrayIndices\" style=\"display: flex; margin: 2px; position: relative\"\r\n            (mouseenter)=\"hover=i\" (mouseleave)=\"hover=null\">\r\n            <mat-form-field>\r\n                <mat-label>{{getLabel()}} key</mat-label>\r\n                <input [disabled]=\"readOnly\" autocomplete=\"off\" matInput [value]=\"additionalPropNames[i]\"\r\n                    (change)=\"fieldNameChange(additionalPropNames[i], $event.target.value)\">\r\n            </mat-form-field>\r\n            &nbsp;\r\n            <lib-json-schema-form #children [label]=\"getLabel() + ' value'\" [value]=\"value[additionalPropNames[i]]\"\r\n                (valueChange)=\"onValueChange(additionalPropNames[i], $event)\" [rootValue]=\"rootValue\"\r\n                [rootSchema]=\"rootSchema\" [schema]=\"schema.additionalProperties\" inArray=\"true\" [base]=\"base\"\r\n                (schemaChange)=\"schemaChange.emit()\">\r\n            </lib-json-schema-form>\r\n            <button *ngIf=\"!readOnly\" mat-icon-button (click)=\"removeField(additionalPropNames[i])\"\r\n                class=\"example-handle\" [style.visibility]=\"i===hover ? 'visible' : 'hidden'\"\r\n                matTooltip=\"{{getLabel()}}\">\r\n                <mat-icon>remove_circle_outline</mat-icon>\r\n            </button>\r\n        </div>\r\n        <button *ngIf=\"!readOnly\" mat-icon-button (click)=\" addField()\" matTooltip=\"{{getLabel()}}\">\r\n            <mat-icon>add_circle_outline</mat-icon>\r\n        </button>\r\n    </span>\r\n\r\n    <!-- additional properties tab layout -->\r\n    <mat-tab-group animationDuration=\"0ms\" *ngIf=\"getLayout() === 'additionalPropertiesTab'\" style=\"max-width: 96vw\"\r\n        [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n        <mat-tab *ngFor=\"let x of value | keyvalue:originalOrder; let i=index\" style=\"margin: 2px; position: relative\">\r\n            <ng-template mat-tab-label>\r\n                <mat-form-field *ngIf=\"!readOnly\">\r\n                    <input autocomplete=\"off\" matInput [value]=\"x.key\"\r\n                        (change)=\"fieldNameChange(x.key, $event.target.value)\">\r\n                </mat-form-field>\r\n                <span *ngIf=\"readOnly\">{{x.key}}</span>\r\n            </ng-template>\r\n            <lib-json-schema-form #children [value]=\"value[x.key]\" (valueChange)=\"onValueChange(x.key, $event)\"\r\n                [rootValue]=\"rootValue\" [rootSchema]=\"rootSchema\" [schema]=\"schema.additionalProperties\" inArray=\"true\"\r\n                [base]=\"base\" (schemaChange)=\"schemaChange.emit()\">\r\n            </lib-json-schema-form>\r\n            <button *ngIf=\"!readOnly\" mat-icon-button (click)=\"removeField(x.key)\" class=\"example-handle\"\r\n                matTooltip=\"{{getLabel()}}\">\r\n                <mat-icon>remove_circle_outline</mat-icon>\r\n            </button>\r\n        </mat-tab>\r\n        <mat-tab *ngIf=\"!readOnly\">\r\n            <ng-template mat-tab-label>\r\n                <button mat-icon-button (click)=\"addField()\" matTooltip=\"{{getLabel()}}\">\r\n                    <mat-icon>add_circle_outline</mat-icon>\r\n                </button>\r\n            </ng-template>\r\n        </mat-tab>\r\n    </mat-tab-group>\r\n\r\n    <!-- array row wrap layout -->\r\n    <span *ngIf=\"getLayout() === 'array'\" [fxLayout]=\"schema.layout === 'vertical' ? 'column' : 'row wrap'\"\r\n        [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n        <div *ngFor=\"let i of arrayIndices\" style=\"display: flex; margin: 2px; position: relative\"\r\n            (mouseenter)=\"hover=i\" (mouseleave)=\"hover=null\">\r\n            <lib-json-schema-form #children [label]=\"getLabel()\" [value]=\"value[i]\"\r\n                (valueChange)=\"setIndexAndEmit(i, $event)\" inArray=\"true\" [rootValue]=\"rootValue\"\r\n                [rootSchema]=\"rootSchema\" [schema]=\"schema.items\" [base]=\"base\" (schemaChange)=\"schemaChange.emit()\">\r\n            </lib-json-schema-form>\r\n            <button *ngIf=\"!readOnly\" mat-icon-button (click)=\"remove(i)\" class=\"example-handle\"\r\n                [style.visibility]=\"i===hover ? 'visible' : 'hidden'\" matTooltip=\"{{getLabel()}}\">\r\n                <mat-icon>remove_circle_outline</mat-icon>\r\n            </button>\r\n        </div>\r\n        <button *ngIf=\"!readOnly\" mat-icon-button (click)=\" add()\" matTooltip=\"{{getLabel()}}\">\r\n            <mat-icon>add_circle_outline</mat-icon>\r\n        </button>\r\n    </span>\r\n\r\n    <!-- array chips layout -->\r\n    <span *ngIf=\"getLayout() === 'chips'\">\r\n        <mat-form-field [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-chip-grid #chipList aria-label=\"Fruit selection\" cdkDropList cdkDropListOrientation=\"horizontal\"\r\n                (cdkDropListDropped)=\"dropChip($event)\">\r\n                <mat-chip-row *ngFor=\"let v of value\" [removable]=\"!readOnly\" (removed)=\"removeChip(v)\" cdkDrag>\r\n                    {{v}}\r\n                    <mat-icon matChipRemove>cancel</mat-icon>\r\n                </mat-chip-row>\r\n                <input [placeholder]=\"schema.title\" autocomplete=\"off\" [matChipInputFor]=\"chipList\"\r\n                    [matChipInputSeparatorKeyCodes]=\"separatorKeysCodes\" (matChipInputTokenEnd)=\"addChip($event)\"\r\n                    [disabled]=\"readOnly\">\r\n            </mat-chip-grid>\r\n        </mat-form-field>\r\n    </span>\r\n\r\n    <!-- array tab layout -->\r\n    <mat-tab-group animationDuration=\"0ms\" *ngIf=\"getLayout() === 'tab'\" style=\"max-width: 96vw\"\r\n        [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n        <mat-tab *ngFor=\"let i of arrayIndices\" style=\"margin: 2px; position: relative\" [label]=\"getLabel()+' '+i\">\r\n            <lib-json-schema-form #children [value]=\"value[i]\" (valueChange)=\"setIndexAndEmit(i, $event)\" inArray=\"true\"\r\n                [rootValue]=\"rootValue\" [rootSchema]=\"rootSchema\" [schema]=\"schema.items\" [base]=\"base\"\r\n                (schemaChange)=\"schemaChange.emit()\">\r\n            </lib-json-schema-form>\r\n            <button *ngIf=\"!readOnly\" mat-icon-button (click)=\"remove(i)\" class=\"example-handle\"\r\n                matTooltip=\"{{getLabel()}}\">\r\n                <mat-icon>remove_circle_outline</mat-icon>\r\n            </button>\r\n        </mat-tab>\r\n        <mat-tab *ngIf=\"!readOnly\" disabled>\r\n            <ng-template mat-tab-label>\r\n                <button mat-icon-button (click)=\" add()\" matTooltip=\"{{getLabel()}}\">\r\n                    <mat-icon style=\"color: black;\">add_circle_outline</mat-icon>\r\n                </button>\r\n            </ng-template>\r\n        </mat-tab>\r\n    </mat-tab-group>\r\n\r\n    <!-- array table layout -->\r\n    <span *ngIf=\"getLayout() === 'table'\" [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n        <table summary=\"Properties table\">\r\n            <tr>\r\n                <th *ngFor=\"let x of schema.items.properties | keyvalue:originalOrder\" scope=\"col\">\r\n                    {{x.value.title ? x.value.title : x.key}}</th>\r\n                <th id=\"remove_row\"></th>\r\n            </tr>\r\n            <tr *ngFor=\"let row of value; let i=index\">\r\n                <td *ngFor=\"let x of schema.items.properties | keyvalue:originalOrder\">\r\n                    <lib-json-schema-form #children [value]=\"row[x.key]\"\r\n                        (valueChange)=\"setIndexAndEmitTable(i, x.key, $event)\" [rootValue]=\"rootValue\"\r\n                        [rootSchema]=\"rootSchema\" [schema]=\"schema.items.properties[x.key]\" [base]=\"base\"\r\n                        (schemaChange)=\"schemaChange.emit()\">\r\n                    </lib-json-schema-form>\r\n                </td>\r\n                <td>\r\n                    <button *ngIf=\"!readOnly\" mat-icon-button (click)=\"remove(i)\" matTooltip=\"{{getLabel()}}\">\r\n                        <mat-icon>remove_circle_outline</mat-icon>\r\n                    </button>\r\n                </td>\r\n            </tr>\r\n        </table>\r\n        <button *ngIf=\"!readOnly\" mat-icon-button (click)=\" add()\" matTooltip=\"{{getLabel()}}\">\r\n            <mat-icon>add_circle_outline</mat-icon>\r\n        </button>\r\n    </span>\r\n\r\n    <!-- enum -->\r\n    <div *ngIf=\"getLayout() === 'enum'\">\r\n        <mat-form-field [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-label>{{label}}</mat-label>\r\n            <mat-select [value]=\"value\" (selectionChange)=\"change($event)\" [disabled]=\"readOnly\">\r\n                <mat-option *ngFor=\"let x of schema.enum\" [value]=\"x\">{{x}}</mat-option>\r\n            </mat-select>\r\n        </mat-form-field>\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- array-select -->\r\n    <div *ngIf=\"getLayout() === 'array-select'\">\r\n        <mat-form-field [matTooltip]=\"schema.description\" [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-label>{{label}}</mat-label>\r\n            <mat-select multiple [value]=\"value\" (focus)=\"focus()\" (selectionChange)=\"change($event)\"\r\n                [disabled]=\"readOnly\">\r\n                <mat-option *ngFor=\"let choice of choices | async\" [value]=\"choice.value\"\r\n                    [matTooltip]=\"choice.name !== choice.value ? choice.value : null\">\r\n                    {{choice.name}}\r\n                </mat-option>\r\n            </mat-select>\r\n        </mat-form-field>\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- date element -->\r\n    <div *ngIf=\"getLayout() === 'date'\">\r\n        <mat-form-field [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-label>{{label}}</mat-label>\r\n            <input autocomplete=\"off\" matInput [value]=\"parseDate(value, schema.dateFormat)\"\r\n                (dateChange)=\"change($event)\" [matDatepicker]=\"picker\" [disabled]=\"readOnly\" style=\"width: 100%\">\r\n            <mat-datepicker-toggle matSuffix [for]=\"picker\"></mat-datepicker-toggle>\r\n            <mat-datepicker #picker></mat-datepicker>\r\n        </mat-form-field>\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- checkbox element -->\r\n    <div *ngIf=\"getLayout() === 'checkbox'\" [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\"\r\n        style=\"margin: 6px; margin-top: 12px;\">\r\n        <mat-checkbox [checked]=\"value\" (change)=\"change($event)\" [disabled]=\"readOnly\">\r\n            {{label}}</mat-checkbox>\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- autocomplete element -->\r\n    <div *ngIf=\"getLayout() === 'autocomplete'\">\r\n        <mat-form-field *ngIf=\"schema.widget === 'select'\" [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-label>{{label}}</mat-label>\r\n            <mat-select [value]=\"value\" (focus)=\"focus()\" (selectionChange)=\"change($event)\" [disabled]=\"readOnly\">\r\n                <mat-option *ngFor=\"let choice of choices | async\" [value]=\"choice.value\"\r\n                    [matTooltip]=\"choice.name !== choice.value ? choice.value : null\">\r\n                    {{choice.name}}\r\n                </mat-option>\r\n            </mat-select>\r\n        </mat-form-field>\r\n        <mat-form-field *ngIf=\"schema.widget !== 'select'\" [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-label>{{label}}</mat-label>\r\n            <input type=\"text\" matInput [formControl]=\"control\" [matAutocomplete]=\"auto\" (focus)=\"focus()\">\r\n            <mat-autocomplete #auto=\"matAutocomplete\">\r\n                <mat-option *ngFor=\"let choice of filteredOptions | async\" [value]=\"choice.value\"\r\n                    [matTooltip]=\"choice.name !== choice.value ? choice.value : null\">\r\n                    {{choice.name}}\r\n                </mat-option>\r\n            </mat-autocomplete>\r\n        </mat-form-field>\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- single element -->\r\n    <div *ngIf=\"getLayout() === 'single'\">\r\n        <mat-form-field [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-label>{{label}}</mat-label>\r\n            <input [disabled]=\"readOnly\" [placeholder]=\"example()\" autocomplete=\"off\" matInput\r\n                [type]=\"getInputType(schema)\" [value]=\"value\" (input)=\"change($event)\"\r\n                [step]=\"getStepValue(schema)\" (focus)=\"getInputType(schema) === 'password' ? value = '':''\">\r\n        </mat-form-field>\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- textarea element -->\r\n    <div *ngIf=\"getLayout() === 'textarea'\">\r\n        <mat-form-field [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-label>{{label}}</mat-label>\r\n            <textarea [disabled]=\"readOnly\" [placeholder]=\"example()\" matInput [value]=\"value\" (input)=\"change($event)\"\r\n                [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\"></textarea>\r\n        </mat-form-field>\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- upload -->\r\n    <div *ngIf=\"getLayout() === 'upload'\">\r\n        <input type=\"file\" id=\"file\" (change)=\"handleFileInput(false, $event)\" [ngStyle]=\"schema.style\"\r\n            [ngClass]=\"schema.class\" [disabled]=\"readOnly\">\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- upload64 -->\r\n    <div *ngIf=\"getLayout() === 'upload64'\">\r\n        <input type=\"file\" id=\"file\" (change)=\"handleFileInput(true, $event)\" [ngStyle]=\"schema.style\"\r\n            [ngClass]=\"schema.class\" [disabled]=\"readOnly\">\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n</div>\r\n<ng-template libWidgetHost></ng-template>", styles: [".example-handle{position:absolute;z-index:9;top:0;right:0}.error{font-size:small;color:red;position:relative;top:-18px}\n"], dependencies: [{ kind: "directive", type: i4.DefaultLayoutDirective, selector: "  [fxLayout], [fxLayout.xs], [fxLayout.sm], [fxLayout.md],  [fxLayout.lg], [fxLayout.xl], [fxLayout.lt-sm], [fxLayout.lt-md],  [fxLayout.lt-lg], [fxLayout.lt-xl], [fxLayout.gt-xs], [fxLayout.gt-sm],  [fxLayout.gt-md], [fxLayout.gt-lg]", inputs: ["fxLayout", "fxLayout.xs", "fxLayout.sm", "fxLayout.md", "fxLayout.lg", "fxLayout.xl", "fxLayout.lt-sm", "fxLayout.lt-md", "fxLayout.lt-lg", "fxLayout.lt-xl", "fxLayout.gt-xs", "fxLayout.gt-sm", "fxLayout.gt-md", "fxLayout.gt-lg"] }, { kind: "directive", type: i5.DefaultClassDirective, selector: "  [ngClass], [ngClass.xs], [ngClass.sm], [ngClass.md], [ngClass.lg], [ngClass.xl],  [ngClass.lt-sm], [ngClass.lt-md], [ngClass.lt-lg], [ngClass.lt-xl],  [ngClass.gt-xs], [ngClass.gt-sm], [ngClass.gt-md], [ngClass.gt-lg]", inputs: ["ngClass", "ngClass.xs", "ngClass.sm", "ngClass.md", "ngClass.lg", "ngClass.xl", "ngClass.lt-sm", "ngClass.lt-md", "ngClass.lt-lg", "ngClass.lt-xl", "ngClass.gt-xs", "ngClass.gt-sm", "ngClass.gt-md", "ngClass.gt-lg"] }, { kind: "directive", type: i5.DefaultStyleDirective, selector: "  [ngStyle],  [ngStyle.xs], [ngStyle.sm], [ngStyle.md], [ngStyle.lg], [ngStyle.xl],  [ngStyle.lt-sm], [ngStyle.lt-md], [ngStyle.lt-lg], [ngStyle.lt-xl],  [ngStyle.gt-xs], [ngStyle.gt-sm], [ngStyle.gt-md], [ngStyle.gt-lg]", inputs: ["ngStyle", "ngStyle.xs", "ngStyle.sm", "ngStyle.md", "ngStyle.lg", "ngStyle.xl", "ngStyle.lt-sm", "ngStyle.lt-md", "ngStyle.lt-lg", "ngStyle.lt-xl", "ngStyle.gt-xs", "ngStyle.gt-sm", "ngStyle.gt-md", "ngStyle.gt-lg"] }, { kind: "component", type: i6.MatIcon, selector: "mat-icon", inputs: ["color", "inline", "svgIcon", "fontSet", "fontIcon"], exportAs: ["matIcon"] }, { kind: "directive", type: i7.MatTooltip, selector: "[matTooltip]", exportAs: ["matTooltip"] }, { kind: "directive", type: i8.MatTabLabel, selector: "[mat-tab-label], [matTabLabel]" }, { kind: "component", type: i8.MatTab, selector: "mat-tab", inputs: ["disabled"], exportAs: ["matTab"] }, { kind: "component", type: i8.MatTabGroup, selector: "mat-tab-group", inputs: ["color", "disableRipple", "fitInkBarToContent", "mat-stretch-tabs"], exportAs: ["matTabGroup"] }, { kind: "component", type: i9.MatFormField, selector: "mat-form-field", inputs: ["hideRequiredMarker", "color", "floatLabel", "appearance", "subscriptSizing", "hintLabel"], exportAs: ["matFormField"] }, { kind: "directive", type: i9.MatLabel, selector: "mat-label" }, { kind: "directive", type: i9.MatSuffix, selector: "[matSuffix], [matIconSuffix], [matTextSuffix]", inputs: ["matTextSuffix"] }, { kind: "component", type: i10.MatSelect, selector: "mat-select", inputs: ["disabled", "disableRipple", "tabIndex"], exportAs: ["matSelect"] }, { kind: "component", type: i11.MatOption, selector: "mat-option", exportAs: ["matOption"] }, { kind: "component", type: i12.MatDatepicker, selector: "mat-datepicker", exportAs: ["matDatepicker"] }, { kind: "directive", type: i12.MatDatepickerInput, selector: "input[matDatepicker]", inputs: ["matDatepicker", "min", "max", "matDatepickerFilter"], exportAs: ["matDatepickerInput"] }, { kind: "component", type: i12.MatDatepickerToggle, selector: "mat-datepicker-toggle", inputs: ["for", "tabIndex", "aria-label", "disabled", "disableRipple"], exportAs: ["matDatepickerToggle"] }, { kind: "component", type: i13.MatCheckbox, selector: "mat-checkbox", inputs: ["disableRipple", "color", "tabIndex"], exportAs: ["matCheckbox"] }, { kind: "component", type: i14.MatAutocomplete, selector: "mat-autocomplete", inputs: ["disableRipple"], exportAs: ["matAutocomplete"] }, { kind: "directive", type: i14.MatAutocompleteTrigger, selector: "input[matAutocomplete], textarea[matAutocomplete]", exportAs: ["matAutocompleteTrigger"] }, { kind: "directive", type: i15.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "directive", type: i15.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i15.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i15.NgStyle, selector: "[ngStyle]", inputs: ["ngStyle"] }, { kind: "directive", type: i16.MatInput, selector: "input[matInput], textarea[matInput], select[matNativeControl],      input[matNativeControl], textarea[matNativeControl]", inputs: ["disabled", "id", "placeholder", "name", "required", "type", "errorStateMatcher", "aria-describedby", "value", "readonly"], exportAs: ["matInput"] }, { kind: "component", type: i17.MatIconButton, selector: "button[mat-icon-button]", inputs: ["disabled", "disableRipple", "color"], exportAs: ["matButton"] }, { kind: "component", type: i18.MatExpansionPanel, selector: "mat-expansion-panel", inputs: ["disabled", "expanded", "hideToggle", "togglePosition"], outputs: ["opened", "closed", "expandedChange", "afterExpand", "afterCollapse"], exportAs: ["matExpansionPanel"] }, { kind: "component", type: i18.MatExpansionPanelHeader, selector: "mat-expansion-panel-header", inputs: ["tabIndex", "expandedHeight", "collapsedHeight"] }, { kind: "directive", type: i18.MatExpansionPanelTitle, selector: "mat-panel-title" }, { kind: "directive", type: i18.MatExpansionPanelDescription, selector: "mat-panel-description" }, { kind: "component", type: i19.MatMenu, selector: "mat-menu", exportAs: ["matMenu"] }, { kind: "component", type: i19.MatMenuItem, selector: "[mat-menu-item]", inputs: ["disabled", "disableRipple", "role"], exportAs: ["matMenuItem"] }, { kind: "directive", type: i19.MatMenuTrigger, selector: "[mat-menu-trigger-for], [matMenuTriggerFor]", exportAs: ["matMenuTrigger"] }, { kind: "directive", type: i20.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { kind: "directive", type: i20.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i20.FormControlDirective, selector: "[formControl]", inputs: ["formControl", "disabled", "ngModel"], outputs: ["ngModelChange"], exportAs: ["ngForm"] }, { kind: "component", type: i21.MatChipGrid, selector: "mat-chip-grid", inputs: ["tabIndex", "disabled", "placeholder", "required", "value", "errorStateMatcher"], outputs: ["change", "valueChange"] }, { kind: "directive", type: i21.MatChipInput, selector: "input[matChipInputFor]", inputs: ["matChipInputFor", "matChipInputAddOnBlur", "matChipInputSeparatorKeyCodes", "placeholder", "id", "disabled"], outputs: ["matChipInputTokenEnd"], exportAs: ["matChipInput", "matChipInputFor"] }, { kind: "directive", type: i21.MatChipRemove, selector: "[matChipRemove]" }, { kind: "component", type: i21.MatChipRow, selector: "mat-chip-row, mat-basic-chip-row", inputs: ["color", "disabled", "disableRipple", "tabIndex", "editable"], outputs: ["edited"] }, { kind: "directive", type: i22.CdkDropList, selector: "[cdkDropList], cdk-drop-list", inputs: ["cdkDropListConnectedTo", "cdkDropListData", "cdkDropListOrientation", "id", "cdkDropListLockAxis", "cdkDropListDisabled", "cdkDropListSortingDisabled", "cdkDropListEnterPredicate", "cdkDropListSortPredicate", "cdkDropListAutoScrollDisabled", "cdkDropListAutoScrollStep"], outputs: ["cdkDropListDropped", "cdkDropListEntered", "cdkDropListExited", "cdkDropListSorted"], exportAs: ["cdkDropList"] }, { kind: "directive", type: i22.CdkDrag, selector: "[cdkDrag]", inputs: ["cdkDragData", "cdkDragLockAxis", "cdkDragRootElement", "cdkDragBoundary", "cdkDragStartDelay", "cdkDragFreeDragPosition", "cdkDragDisabled", "cdkDragConstrainPosition", "cdkDragPreviewClass", "cdkDragPreviewContainer"], outputs: ["cdkDragStarted", "cdkDragReleased", "cdkDragEnded", "cdkDragEntered", "cdkDragExited", "cdkDragDropped", "cdkDragMoved"], exportAs: ["cdkDrag"] }, { kind: "component", type: JsonSchemaFormComponent, selector: "lib-json-schema-form", inputs: ["name", "label", "value", "rootValue", "schema", "parentSchema", "rootSchema", "base", "switch", "hideUndefined", "inExpansion", "inArray", "required"], outputs: ["valueChange", "errorChange", "schemaChange"] }, { kind: "directive", type: WidgetDirective, selector: "[libWidgetHost]" }, { kind: "pipe", type: i15.AsyncPipe, name: "async" }, { kind: "pipe", type: i15.KeyValuePipe, name: "keyvalue" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.1.2", ngImport: i0, type: JsonSchemaFormComponent, decorators: [{
            type: Component,
            args: [{ selector: 'lib-json-schema-form', template: "<mat-expansion-panel *ngIf=\"(schema.expanded === true || schema.expanded === false) && !inExpansion\"\r\n    [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\" [expanded]=\"schema.expanded\">\r\n    <mat-expansion-panel-header>\r\n        <mat-panel-title>{{label}}</mat-panel-title>\r\n        <mat-panel-description>{{schema.description}}</mat-panel-description>\r\n    </mat-expansion-panel-header>\r\n    <lib-json-schema-form #child [label]=\"label\" [name]=\"name\" [value]=\"value\" (valueChange)=\"setAndEmit($event)\"\r\n        [switch]=\"switch\" [rootValue]=\"rootValue\" [rootSchema]=\"rootSchema\" [schema]=\"schema\" [inExpansion]=\"true\"\r\n        [base]=\"base\" (schemaChange)=\"schemaChange.emit()\">\r\n    </lib-json-schema-form>\r\n</mat-expansion-panel>\r\n\r\n<div *ngIf=\"(schema.expanded === null || schema.expanded === undefined) || inExpansion\"\r\n    [matTooltip]=\"schema.description\" role=\"form\">\r\n\r\n    <div *ngIf=\"service.editMode && !schema.static && !inArray && getLayout() !== 'none'\">\r\n        <mat-icon style=\"cursor: pointer\" [matMenuTriggerFor]=\"menu\"\r\n            [matTooltip]=\"name ? 'Form element '+name : (isRoot ? 'Form root' : '')\">\r\n            more_vert\r\n        </mat-icon>\r\n\r\n        <mat-menu #menu=\"matMenu\">\r\n            <button mat-menu-item (click)=\"edit.edit()\">\r\n                <mat-icon>edit</mat-icon>Edit\r\n            </button>\r\n            <button [disabled]=\"!edit.canAdd()\" mat-menu-item [matMenuTriggerFor]=\"madd\">\r\n                <mat-icon>add</mat-icon>Show\r\n            </button>\r\n            <button [disabled]=\"!edit.canAddSub()\" mat-menu-item [matMenuTriggerFor]=\"madd2\">\r\n                <mat-icon>playlist_add</mat-icon>Show inline\r\n            </button>\r\n            <button [disabled]=\" !edit.canHide()\" mat-menu-item (click)=\"edit.hide()\">\r\n                <mat-icon>remove</mat-icon>Hide\r\n            </button>\r\n            <button [disabled]=\"!edit.canUp()\" mat-menu-item (click)=\"edit.up()\">\r\n                <mat-icon>north_west</mat-icon>Up / Left\r\n            </button>\r\n            <button [disabled]=\"!edit.canDown()\" mat-menu-item (click)=\"edit.down()\">\r\n                <mat-icon>south_east</mat-icon>Down / Right\r\n            </button>\r\n        </mat-menu>\r\n\r\n        <mat-menu #madd=\"matMenu\">\r\n            <button *ngFor=\"let x of edit.addable()\" mat-menu-item (click)=\"edit.add(x)\">{{x}}</button>\r\n        </mat-menu>\r\n\r\n        <mat-menu #madd2=\"matMenu\">\r\n            <button *ngFor=\"let x of edit.subaddable()\" mat-menu-item (click)=\"edit.addSub(x)\">{{x}}</button>\r\n        </mat-menu>\r\n    </div>\r\n\r\n    <!-- object row wrap layout -->\r\n    <span *ngIf=\"getLayout() === 'object'\" [fxLayout]=\"schema.layout === 'vertical' ? 'column' : 'row wrap'\"\r\n        [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n        <div *ngFor=\"let op of orderedProperties\" [fxLayout]=\"schema.layout !== 'vertical' ? 'column' : 'row wrap'\">\r\n            <div *ngFor=\" let x of op | keyvalue:originalOrder\" style=\"display: flex; margin: 2px\">\r\n                <div *ngIf=\"x.value && (x.value.type || x.value.$ref)\">\r\n                    <lib-json-schema-form #children [name]=\"x.key\" [label]=\"x.value.title ? x.value.title : x.key\"\r\n                        [value]=\"value ? value[x.key] : undefined\" (valueChange)=\"onValueChange(x.key, $event)\"\r\n                        [switch]=\"value ? value[schema.switch] : undefined\" [rootValue]=\"rootValue\"\r\n                        [rootSchema]=\"rootSchema\" [schema]=\"x.value\" [parentSchema]=\"schema\"\r\n                        [required]=\"schema.required ? schema.required.includes(x.key) : false\"\r\n                        [hideUndefined]=\"schema.hideUndefined\" [base]=\"base\" (schemaChange)=\"schemaChange.emit()\">\r\n                    </lib-json-schema-form>\r\n                </div>\r\n            </div>\r\n        </div>\r\n        <div *ngIf=\"schema.hideUndefined\" style=\"margin-right: 20px;\">\r\n            <button mat-icon-button [matMenuTriggerFor]=\"menu\">\r\n                <mat-icon>more_vert</mat-icon>\r\n            </button>\r\n            <mat-menu #menu=\"matMenu\">\r\n                <div *ngFor=\"let x of showPropertyList()\">\r\n                    <mat-checkbox [disabled]=\"value ? value[x] : false\" [checked]=\"value && (value[x] !== undefined)\"\r\n                        (change)=\"showProperty(x)\">\r\n                        {{x}}\r\n                    </mat-checkbox>\r\n                </div>\r\n            </mat-menu>\r\n        </div>\r\n    </span>\r\n\r\n    <!-- additional properties -->\r\n    <span *ngIf=\"getLayout() === 'additionalProperties'\"\r\n        [fxLayout]=\"schema.layout === 'vertical' ? 'column' : 'row wrap'\" [ngStyle]=\"schema.style\"\r\n        [ngClass]=\"schema.class\">\r\n        <div *ngFor=\"let i of arrayIndices\" style=\"display: flex; margin: 2px; position: relative\"\r\n            (mouseenter)=\"hover=i\" (mouseleave)=\"hover=null\">\r\n            <mat-form-field>\r\n                <mat-label>{{getLabel()}} key</mat-label>\r\n                <input [disabled]=\"readOnly\" autocomplete=\"off\" matInput [value]=\"additionalPropNames[i]\"\r\n                    (change)=\"fieldNameChange(additionalPropNames[i], $event.target.value)\">\r\n            </mat-form-field>\r\n            &nbsp;\r\n            <lib-json-schema-form #children [label]=\"getLabel() + ' value'\" [value]=\"value[additionalPropNames[i]]\"\r\n                (valueChange)=\"onValueChange(additionalPropNames[i], $event)\" [rootValue]=\"rootValue\"\r\n                [rootSchema]=\"rootSchema\" [schema]=\"schema.additionalProperties\" inArray=\"true\" [base]=\"base\"\r\n                (schemaChange)=\"schemaChange.emit()\">\r\n            </lib-json-schema-form>\r\n            <button *ngIf=\"!readOnly\" mat-icon-button (click)=\"removeField(additionalPropNames[i])\"\r\n                class=\"example-handle\" [style.visibility]=\"i===hover ? 'visible' : 'hidden'\"\r\n                matTooltip=\"{{getLabel()}}\">\r\n                <mat-icon>remove_circle_outline</mat-icon>\r\n            </button>\r\n        </div>\r\n        <button *ngIf=\"!readOnly\" mat-icon-button (click)=\" addField()\" matTooltip=\"{{getLabel()}}\">\r\n            <mat-icon>add_circle_outline</mat-icon>\r\n        </button>\r\n    </span>\r\n\r\n    <!-- additional properties tab layout -->\r\n    <mat-tab-group animationDuration=\"0ms\" *ngIf=\"getLayout() === 'additionalPropertiesTab'\" style=\"max-width: 96vw\"\r\n        [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n        <mat-tab *ngFor=\"let x of value | keyvalue:originalOrder; let i=index\" style=\"margin: 2px; position: relative\">\r\n            <ng-template mat-tab-label>\r\n                <mat-form-field *ngIf=\"!readOnly\">\r\n                    <input autocomplete=\"off\" matInput [value]=\"x.key\"\r\n                        (change)=\"fieldNameChange(x.key, $event.target.value)\">\r\n                </mat-form-field>\r\n                <span *ngIf=\"readOnly\">{{x.key}}</span>\r\n            </ng-template>\r\n            <lib-json-schema-form #children [value]=\"value[x.key]\" (valueChange)=\"onValueChange(x.key, $event)\"\r\n                [rootValue]=\"rootValue\" [rootSchema]=\"rootSchema\" [schema]=\"schema.additionalProperties\" inArray=\"true\"\r\n                [base]=\"base\" (schemaChange)=\"schemaChange.emit()\">\r\n            </lib-json-schema-form>\r\n            <button *ngIf=\"!readOnly\" mat-icon-button (click)=\"removeField(x.key)\" class=\"example-handle\"\r\n                matTooltip=\"{{getLabel()}}\">\r\n                <mat-icon>remove_circle_outline</mat-icon>\r\n            </button>\r\n        </mat-tab>\r\n        <mat-tab *ngIf=\"!readOnly\">\r\n            <ng-template mat-tab-label>\r\n                <button mat-icon-button (click)=\"addField()\" matTooltip=\"{{getLabel()}}\">\r\n                    <mat-icon>add_circle_outline</mat-icon>\r\n                </button>\r\n            </ng-template>\r\n        </mat-tab>\r\n    </mat-tab-group>\r\n\r\n    <!-- array row wrap layout -->\r\n    <span *ngIf=\"getLayout() === 'array'\" [fxLayout]=\"schema.layout === 'vertical' ? 'column' : 'row wrap'\"\r\n        [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n        <div *ngFor=\"let i of arrayIndices\" style=\"display: flex; margin: 2px; position: relative\"\r\n            (mouseenter)=\"hover=i\" (mouseleave)=\"hover=null\">\r\n            <lib-json-schema-form #children [label]=\"getLabel()\" [value]=\"value[i]\"\r\n                (valueChange)=\"setIndexAndEmit(i, $event)\" inArray=\"true\" [rootValue]=\"rootValue\"\r\n                [rootSchema]=\"rootSchema\" [schema]=\"schema.items\" [base]=\"base\" (schemaChange)=\"schemaChange.emit()\">\r\n            </lib-json-schema-form>\r\n            <button *ngIf=\"!readOnly\" mat-icon-button (click)=\"remove(i)\" class=\"example-handle\"\r\n                [style.visibility]=\"i===hover ? 'visible' : 'hidden'\" matTooltip=\"{{getLabel()}}\">\r\n                <mat-icon>remove_circle_outline</mat-icon>\r\n            </button>\r\n        </div>\r\n        <button *ngIf=\"!readOnly\" mat-icon-button (click)=\" add()\" matTooltip=\"{{getLabel()}}\">\r\n            <mat-icon>add_circle_outline</mat-icon>\r\n        </button>\r\n    </span>\r\n\r\n    <!-- array chips layout -->\r\n    <span *ngIf=\"getLayout() === 'chips'\">\r\n        <mat-form-field [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-chip-grid #chipList aria-label=\"Fruit selection\" cdkDropList cdkDropListOrientation=\"horizontal\"\r\n                (cdkDropListDropped)=\"dropChip($event)\">\r\n                <mat-chip-row *ngFor=\"let v of value\" [removable]=\"!readOnly\" (removed)=\"removeChip(v)\" cdkDrag>\r\n                    {{v}}\r\n                    <mat-icon matChipRemove>cancel</mat-icon>\r\n                </mat-chip-row>\r\n                <input [placeholder]=\"schema.title\" autocomplete=\"off\" [matChipInputFor]=\"chipList\"\r\n                    [matChipInputSeparatorKeyCodes]=\"separatorKeysCodes\" (matChipInputTokenEnd)=\"addChip($event)\"\r\n                    [disabled]=\"readOnly\">\r\n            </mat-chip-grid>\r\n        </mat-form-field>\r\n    </span>\r\n\r\n    <!-- array tab layout -->\r\n    <mat-tab-group animationDuration=\"0ms\" *ngIf=\"getLayout() === 'tab'\" style=\"max-width: 96vw\"\r\n        [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n        <mat-tab *ngFor=\"let i of arrayIndices\" style=\"margin: 2px; position: relative\" [label]=\"getLabel()+' '+i\">\r\n            <lib-json-schema-form #children [value]=\"value[i]\" (valueChange)=\"setIndexAndEmit(i, $event)\" inArray=\"true\"\r\n                [rootValue]=\"rootValue\" [rootSchema]=\"rootSchema\" [schema]=\"schema.items\" [base]=\"base\"\r\n                (schemaChange)=\"schemaChange.emit()\">\r\n            </lib-json-schema-form>\r\n            <button *ngIf=\"!readOnly\" mat-icon-button (click)=\"remove(i)\" class=\"example-handle\"\r\n                matTooltip=\"{{getLabel()}}\">\r\n                <mat-icon>remove_circle_outline</mat-icon>\r\n            </button>\r\n        </mat-tab>\r\n        <mat-tab *ngIf=\"!readOnly\" disabled>\r\n            <ng-template mat-tab-label>\r\n                <button mat-icon-button (click)=\" add()\" matTooltip=\"{{getLabel()}}\">\r\n                    <mat-icon style=\"color: black;\">add_circle_outline</mat-icon>\r\n                </button>\r\n            </ng-template>\r\n        </mat-tab>\r\n    </mat-tab-group>\r\n\r\n    <!-- array table layout -->\r\n    <span *ngIf=\"getLayout() === 'table'\" [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n        <table summary=\"Properties table\">\r\n            <tr>\r\n                <th *ngFor=\"let x of schema.items.properties | keyvalue:originalOrder\" scope=\"col\">\r\n                    {{x.value.title ? x.value.title : x.key}}</th>\r\n                <th id=\"remove_row\"></th>\r\n            </tr>\r\n            <tr *ngFor=\"let row of value; let i=index\">\r\n                <td *ngFor=\"let x of schema.items.properties | keyvalue:originalOrder\">\r\n                    <lib-json-schema-form #children [value]=\"row[x.key]\"\r\n                        (valueChange)=\"setIndexAndEmitTable(i, x.key, $event)\" [rootValue]=\"rootValue\"\r\n                        [rootSchema]=\"rootSchema\" [schema]=\"schema.items.properties[x.key]\" [base]=\"base\"\r\n                        (schemaChange)=\"schemaChange.emit()\">\r\n                    </lib-json-schema-form>\r\n                </td>\r\n                <td>\r\n                    <button *ngIf=\"!readOnly\" mat-icon-button (click)=\"remove(i)\" matTooltip=\"{{getLabel()}}\">\r\n                        <mat-icon>remove_circle_outline</mat-icon>\r\n                    </button>\r\n                </td>\r\n            </tr>\r\n        </table>\r\n        <button *ngIf=\"!readOnly\" mat-icon-button (click)=\" add()\" matTooltip=\"{{getLabel()}}\">\r\n            <mat-icon>add_circle_outline</mat-icon>\r\n        </button>\r\n    </span>\r\n\r\n    <!-- enum -->\r\n    <div *ngIf=\"getLayout() === 'enum'\">\r\n        <mat-form-field [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-label>{{label}}</mat-label>\r\n            <mat-select [value]=\"value\" (selectionChange)=\"change($event)\" [disabled]=\"readOnly\">\r\n                <mat-option *ngFor=\"let x of schema.enum\" [value]=\"x\">{{x}}</mat-option>\r\n            </mat-select>\r\n        </mat-form-field>\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- array-select -->\r\n    <div *ngIf=\"getLayout() === 'array-select'\">\r\n        <mat-form-field [matTooltip]=\"schema.description\" [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-label>{{label}}</mat-label>\r\n            <mat-select multiple [value]=\"value\" (focus)=\"focus()\" (selectionChange)=\"change($event)\"\r\n                [disabled]=\"readOnly\">\r\n                <mat-option *ngFor=\"let choice of choices | async\" [value]=\"choice.value\"\r\n                    [matTooltip]=\"choice.name !== choice.value ? choice.value : null\">\r\n                    {{choice.name}}\r\n                </mat-option>\r\n            </mat-select>\r\n        </mat-form-field>\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- date element -->\r\n    <div *ngIf=\"getLayout() === 'date'\">\r\n        <mat-form-field [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-label>{{label}}</mat-label>\r\n            <input autocomplete=\"off\" matInput [value]=\"parseDate(value, schema.dateFormat)\"\r\n                (dateChange)=\"change($event)\" [matDatepicker]=\"picker\" [disabled]=\"readOnly\" style=\"width: 100%\">\r\n            <mat-datepicker-toggle matSuffix [for]=\"picker\"></mat-datepicker-toggle>\r\n            <mat-datepicker #picker></mat-datepicker>\r\n        </mat-form-field>\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- checkbox element -->\r\n    <div *ngIf=\"getLayout() === 'checkbox'\" [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\"\r\n        style=\"margin: 6px; margin-top: 12px;\">\r\n        <mat-checkbox [checked]=\"value\" (change)=\"change($event)\" [disabled]=\"readOnly\">\r\n            {{label}}</mat-checkbox>\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- autocomplete element -->\r\n    <div *ngIf=\"getLayout() === 'autocomplete'\">\r\n        <mat-form-field *ngIf=\"schema.widget === 'select'\" [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-label>{{label}}</mat-label>\r\n            <mat-select [value]=\"value\" (focus)=\"focus()\" (selectionChange)=\"change($event)\" [disabled]=\"readOnly\">\r\n                <mat-option *ngFor=\"let choice of choices | async\" [value]=\"choice.value\"\r\n                    [matTooltip]=\"choice.name !== choice.value ? choice.value : null\">\r\n                    {{choice.name}}\r\n                </mat-option>\r\n            </mat-select>\r\n        </mat-form-field>\r\n        <mat-form-field *ngIf=\"schema.widget !== 'select'\" [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-label>{{label}}</mat-label>\r\n            <input type=\"text\" matInput [formControl]=\"control\" [matAutocomplete]=\"auto\" (focus)=\"focus()\">\r\n            <mat-autocomplete #auto=\"matAutocomplete\">\r\n                <mat-option *ngFor=\"let choice of filteredOptions | async\" [value]=\"choice.value\"\r\n                    [matTooltip]=\"choice.name !== choice.value ? choice.value : null\">\r\n                    {{choice.name}}\r\n                </mat-option>\r\n            </mat-autocomplete>\r\n        </mat-form-field>\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- single element -->\r\n    <div *ngIf=\"getLayout() === 'single'\">\r\n        <mat-form-field [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-label>{{label}}</mat-label>\r\n            <input [disabled]=\"readOnly\" [placeholder]=\"example()\" autocomplete=\"off\" matInput\r\n                [type]=\"getInputType(schema)\" [value]=\"value\" (input)=\"change($event)\"\r\n                [step]=\"getStepValue(schema)\" (focus)=\"getInputType(schema) === 'password' ? value = '':''\">\r\n        </mat-form-field>\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- textarea element -->\r\n    <div *ngIf=\"getLayout() === 'textarea'\">\r\n        <mat-form-field [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\">\r\n            <mat-label>{{label}}</mat-label>\r\n            <textarea [disabled]=\"readOnly\" [placeholder]=\"example()\" matInput [value]=\"value\" (input)=\"change($event)\"\r\n                [ngStyle]=\"schema.style\" [ngClass]=\"schema.class\"></textarea>\r\n        </mat-form-field>\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- upload -->\r\n    <div *ngIf=\"getLayout() === 'upload'\">\r\n        <input type=\"file\" id=\"file\" (change)=\"handleFileInput(false, $event)\" [ngStyle]=\"schema.style\"\r\n            [ngClass]=\"schema.class\" [disabled]=\"readOnly\">\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n\r\n    <!-- upload64 -->\r\n    <div *ngIf=\"getLayout() === 'upload64'\">\r\n        <input type=\"file\" id=\"file\" (change)=\"handleFileInput(true, $event)\" [ngStyle]=\"schema.style\"\r\n            [ngClass]=\"schema.class\" [disabled]=\"readOnly\">\r\n        <p class=\"error mat-typography\">{{error()}}</p>\r\n    </div>\r\n</div>\r\n<ng-template libWidgetHost></ng-template>", styles: [".example-handle{position:absolute;z-index:9;top:0;right:0}.error{font-size:small;color:red;position:relative;top:-18px}\n"] }]
        }], ctorParameters: function () { return [{ type: i1.HttpClient }, { type: i0.ComponentFactoryResolver }, { type: JsonSchemaFormService }, { type: i1$1.MatDialog }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [EDIT_DIALOG_TOKEN]
                }] }]; }, propDecorators: { children: [{
                type: ViewChildren,
                args: ['children']
            }], child: [{
                type: ViewChild,
                args: ['child']
            }], name: [{
                type: Input
            }], label: [{
                type: Input
            }], value: [{
                type: Input
            }], rootValue: [{
                type: Input
            }], valueChange: [{
                type: Output
            }], errorChange: [{
                type: Output
            }], schemaChange: [{
                type: Output
            }], schema: [{
                type: Input
            }], parentSchema: [{
                type: Input
            }], rootSchema: [{
                type: Input
            }], base: [{
                type: Input
            }], switch: [{
                type: Input
            }], hideUndefined: [{
                type: Input
            }], inExpansion: [{
                type: Input
            }], inArray: [{
                type: Input
            }], required: [{
                type: Input
            }], widgetHost: [{
                type: ViewChild,
                args: [WidgetDirective, { static: true }]
            }] } });

/**
 * dialog to edit a JSON node in a layout structure
 */
class EditElementDialogComponent {
    /**
     * dialog constrcutor
     * @param dialogRef   disloag ref
     * @param data        data to edit
     */
    constructor(dialogRef, data) {
        this.dialogRef = dialogRef;
        this.data = data;
        this.schema = {
            title: 'Edit form',
            layout: 'vertical',
            order: [
                ['widget', 'layout', 'itemlayout'],
                ['title', 'description'],
                ['example', 'readOnly'],
                ['required', 'format', 'errorMessage'],
                'choices', 'class', 'style'
            ],
            static: true,
            type: 'object',
            properties: {
                // description, class, style are always applicable
                description: { type: 'string', static: true, title: 'Description / tooltip' },
                class: { type: 'array', items: { type: 'string' }, layout: 'chips', style: { width: '365px' }, static: true, title: 'CSS classes' },
                style: { type: 'object', additionalProperties: { type: 'string' }, static: true, title: 'CSS styles' }
            }
        };
        if (data.properties) {
            // only show required and error message for objects
            this.schema.properties.errorMessage = { type: 'string', static: true, title: 'Validation error message' };
            this.schema.properties.required = {
                choices: Object.keys(data.properties), type: 'array', layout: 'select', items: { type: 'string' }, static: true,
                title: 'Required fields'
            };
            this.schema.properties.layout = { type: 'string', enum: ['vertical', 'horizontal'], static: true, title: 'Screen layout' };
        }
        else {
            if (data.items) {
                this.schema.properties.layout = {
                    type: 'string', enum: ['vertical', 'horizontal', 'select', 'tab', 'table', 'chips'],
                    static: true, title: 'Screen Layout'
                };
                this.schema.properties.itemlayout = { type: 'string', enum: ['vertical', 'horizontal'], static: true, title: 'Item screen layout' };
            }
            if (data.items?.properties) {
                // array of objects
                this.schema.properties.errorMessage = { type: 'string', static: true, title: 'Validation error message' };
                this.schema.properties.required = {
                    choices: Object.keys(data.items.properties), type: 'array', layout: 'select', items: { type: 'string' }, static: true,
                    title: 'Required fields'
                };
            }
            else {
                // array of simple types
                this.schema.properties.readOnly = {
                    type: 'boolean', static: true, title: 'Read only value', style: { 'padding-top': '15px' }
                };
                this.schema.properties.errorMessage = { type: 'string', static: true, title: 'Validation error message' };
                this.schema.properties.format = {
                    type: 'string', static: true, title: 'Format',
                    widget: 'select', choices: [null, 'email', 'ipv4', 'url', 'uri']
                };
                this.schema.properties.example = { type: 'string', static: true, title: 'Example data' };
                this.schema.properties.title = { type: 'string', static: true, title: 'Title' };
                this.schema.properties.widget = {
                    type: 'string', enum: ['text', 'select', 'date', 'textarea', 'password'], static: true,
                    title: 'Form widget'
                };
                this.schema.properties.choices = {
                    type: 'array', items: { type: 'string' }, style: { width: '365px' },
                    layout: 'chips', static: true, title: 'Input choices'
                };
            }
        }
    }
}
EditElementDialogComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.1.2", ngImport: i0, type: EditElementDialogComponent, deps: [{ token: i1$1.MatDialogRef }, { token: MAT_DIALOG_DATA }], target: i0.ɵɵFactoryTarget.Component });
EditElementDialogComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.1.2", type: EditElementDialogComponent, selector: "lib-edit-element-dialog", ngImport: i0, template: "<mat-dialog-content style=\"padding-top: 10px;\">\n    <lib-json-schema-form [(value)]=\"data\" [schema]=\"schema\" [label]=\"schema.title\"></lib-json-schema-form>\n</mat-dialog-content>\n<mat-dialog-actions align=\"end\">\n    <button mat-button (click)=\"dialogRef.close()\">Cancel</button>\n    <button mat-raised-button [mat-dialog-close]=\"data\" cdkFocusInitial>Ok</button>\n</mat-dialog-actions>", dependencies: [{ kind: "component", type: i17.MatButton, selector: "    button[mat-button], button[mat-raised-button], button[mat-flat-button],    button[mat-stroked-button]  ", inputs: ["disabled", "disableRipple", "color"], exportAs: ["matButton"] }, { kind: "directive", type: i1$1.MatDialogClose, selector: "[mat-dialog-close], [matDialogClose]", inputs: ["aria-label", "type", "mat-dialog-close", "matDialogClose"], exportAs: ["matDialogClose"] }, { kind: "directive", type: i1$1.MatDialogContent, selector: "[mat-dialog-content], mat-dialog-content, [matDialogContent]" }, { kind: "directive", type: i1$1.MatDialogActions, selector: "[mat-dialog-actions], mat-dialog-actions, [matDialogActions]", inputs: ["align"] }, { kind: "component", type: JsonSchemaFormComponent, selector: "lib-json-schema-form", inputs: ["name", "label", "value", "rootValue", "schema", "parentSchema", "rootSchema", "base", "switch", "hideUndefined", "inExpansion", "inArray", "required"], outputs: ["valueChange", "errorChange", "schemaChange"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.1.2", ngImport: i0, type: EditElementDialogComponent, decorators: [{
            type: Component,
            args: [{ selector: 'lib-edit-element-dialog', template: "<mat-dialog-content style=\"padding-top: 10px;\">\n    <lib-json-schema-form [(value)]=\"data\" [schema]=\"schema\" [label]=\"schema.title\"></lib-json-schema-form>\n</mat-dialog-content>\n<mat-dialog-actions align=\"end\">\n    <button mat-button (click)=\"dialogRef.close()\">Cancel</button>\n    <button mat-raised-button [mat-dialog-close]=\"data\" cdkFocusInitial>Ok</button>\n</mat-dialog-actions>" }]
        }], ctorParameters: function () { return [{ type: i1$1.MatDialogRef }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [MAT_DIALOG_DATA]
                }] }]; } });

class JsonSchemaFormModule {
}
JsonSchemaFormModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.1.2", ngImport: i0, type: JsonSchemaFormModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
JsonSchemaFormModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "15.1.2", ngImport: i0, type: JsonSchemaFormModule, declarations: [JsonSchemaFormComponent, EditElementDialogComponent, WidgetDirective], imports: [MatCardModule,
        FlexLayoutModule,
        MatIconModule,
        MatTooltipModule,
        MatTableModule,
        MatTabsModule,
        MatFormFieldModule,
        MatSelectModule,
        MatDatepickerModule,
        MatCheckboxModule,
        MatAutocompleteModule,
        HttpClientModule,
        CommonModule,
        MatInputModule,
        MatNativeDateModule,
        MatButtonModule,
        MatExpansionModule,
        MatMenuModule,
        ReactiveFormsModule,
        MatDialogModule,
        MatChipsModule,
        DragDropModule], exports: [JsonSchemaFormComponent] });
JsonSchemaFormModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "15.1.2", ngImport: i0, type: JsonSchemaFormModule, providers: [
        // turn off tooltip gestures on mobile: https://github.com/angular/components/issues/4892
        { provide: MAT_TOOLTIP_DEFAULT_OPTIONS, useValue: { touchGestures: 'off' } },
        { provide: EDIT_DIALOG_TOKEN, useValue: EditElementDialogComponent }
    ], imports: [MatCardModule,
        FlexLayoutModule,
        MatIconModule,
        MatTooltipModule,
        MatTableModule,
        MatTabsModule,
        MatFormFieldModule,
        MatSelectModule,
        MatDatepickerModule,
        MatCheckboxModule,
        MatAutocompleteModule,
        HttpClientModule,
        CommonModule,
        MatInputModule,
        MatNativeDateModule,
        MatButtonModule,
        MatExpansionModule,
        MatMenuModule,
        ReactiveFormsModule,
        MatDialogModule,
        MatChipsModule,
        DragDropModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.1.2", ngImport: i0, type: JsonSchemaFormModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [JsonSchemaFormComponent, EditElementDialogComponent, WidgetDirective],
                    imports: [
                        MatCardModule,
                        FlexLayoutModule,
                        MatIconModule,
                        MatTooltipModule,
                        MatTableModule,
                        MatTabsModule,
                        MatFormFieldModule,
                        MatSelectModule,
                        MatDatepickerModule,
                        MatCheckboxModule,
                        MatAutocompleteModule,
                        HttpClientModule,
                        CommonModule,
                        MatInputModule,
                        MatNativeDateModule,
                        MatButtonModule,
                        MatExpansionModule,
                        MatMenuModule,
                        ReactiveFormsModule,
                        MatDialogModule,
                        MatChipsModule,
                        DragDropModule
                    ],
                    exports: [JsonSchemaFormComponent],
                    providers: [
                        // turn off tooltip gestures on mobile: https://github.com/angular/components/issues/4892
                        { provide: MAT_TOOLTIP_DEFAULT_OPTIONS, useValue: { touchGestures: 'off' } },
                        { provide: EDIT_DIALOG_TOKEN, useValue: EditElementDialogComponent }
                    ]
                }]
        }] });

/*
 * Public API Surface of json-schema-form
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DefaultChoiceHandler, JsonSchemaFormComponent, JsonSchemaFormModule, JsonSchemaFormService };
//# sourceMappingURL=tbotlabs-json-schema-form.mjs.map
